=== modified file 'Makefile'
--- Makefile	2010-04-06 06:59:03 +0000
+++ Makefile	2010-08-30 21:23:49 +0000
@@ -39,8 +39,14 @@
 check: docs check-nodocs
 
 check-nodocs: extensions
+	set -e
 	# Generate a stream for PQM to watch.
+	-$(RM) -f selftest.log
 	$(PYTHON) -Werror -O ./bzr selftest --subunit $(tests) | tee selftest.log
+	# An empty log file should catch errors in the $(PYTHON)
+	# command above (the '|' swallow any errors since 'make'
+	# sees the 'tee' exit code for the whole line
+	if [ ! -s selftest.log ] ; then exit 1 ; fi
 	# Check that there were no errors reported.
 	subunit-stats < selftest.log
 

=== modified file 'NEWS'
--- NEWS	2010-08-31 13:51:03 +0000
+++ NEWS	2010-08-31 14:06:56 +0000
@@ -63,6 +63,15 @@
   to register custom handlers which will be invoked before/after the
   standard status output is displayed. (Parth Malwankar)
 
+* ``bzr break-lock --config [location]`` can now break config files
+  locks. (Vincent Ladeuil, #525571)
+
+* ``bzrlib.config.LockableConfig`` is a base class for config files that
+  needs to be protected against multiple writers. All methods that
+  change a configuration variable value must be decorated with
+  @needs_write_lock (set_option() for example).
+  (Vincent Ladeuil,  #525571)
+
 * The ``lp:`` prefix will now use your known username (from
   ``bzr launchpad-login``) to expand ``~`` to your username.  For example:
   ``bzr launchpad-login user && bzr push lp:~/project/branch`` will now
@@ -91,6 +100,9 @@
 * CommitBuilder now uses the committer instead of _config.username to generate
   the revision-id.  (Aaron Bentley, #614404)
 
+* Configuration files in ``${BZR_HOME}`` are now protected against
+  concurrent writers by using a lock. (Vincent Ladeuil, #525571)
+
 * Cope with Microsoft FTP Server and VSFTPd that return reply '250
   Directory created' when mkdir succeeds.  (Martin Pool, #224373)
 
@@ -103,16 +115,29 @@
 * Don't print internal object name when print an invalid revision spec
   error.  (Neil Martinsen-Burrell, #598701)
 
+* Errors occurring during http(s) test server starts should now be
+  handled cleanly. (Vincent Ladeuil, #392402)
+
 * Fix ``AttributeError on parent.children`` when adding a file under a 
   directory that was a symlink in the previous commit.
   (Martin Pool, #192859)
 
+* Fix spurious paramiko warning on hardy by ensuring that ``selftest``
+  properly remove its warning filter. (Vincent Ladeuil, #625686)
+
 * ``HTTP/1.1`` test servers now set a ``Content-Length`` header to comply
   with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
 
+* Most of the leaked threads during selftest are now fixed, allowing the
+  full test suite to pass on gentoo.
+  (Vincent Ladeuil, #392127)
+
 * `PathNotChild` should not give a traceback.
   (Martin Pool, #98735)
 
+* ``PQM`` will no longer ignore syntax errors in submissions.
+  (Vincent Ladeuil, #626667)
+
 * Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
   which can result in "missing referenced chk root keys" errors when
   fetching from repositories with affected revisions.
@@ -171,6 +196,16 @@
 API Changes
 ***********
 
+* Configuration files should now use the ``from_string`` constructor rather
+  than the ``file`` parameter of the ``_get_parser`` method. The later has
+  been deprecated. ``from_string`` also accept a ``save=True`` parameter to
+  have the configuration file immediately written to disk. 
+  (Vincent Ladeuil)
+
+* ``IniBaseConfig`` objects should now use the ``from_string`` constructor
+  the rather than the ``file`` parameter of the ``_get_parser`` method. The
+  later has been deprecated. (Vincent Ladeuil)
+
 * InventoryEntry instances now raise AttributeError if you try to assign
   to attributes that are irrelevant to that kind of entry.  e.g. setting
   ``symlink_target`` on an InventoryFile will fail.  It is still okay to
@@ -183,6 +218,10 @@
 * InventoryEntry objects no longer have ``_put_in_tar`` or
   ``_put_on_disk`` methods.  (Andrew Bennetts)
 
+* The ``get_filename`` parameter in the ``config.IniBaseConfig``
+  constructor has been deprecated, use the ``file_name`` parameter instead.
+  (Vincent Ladeuil)
+
 Internals
 *********
 
@@ -192,6 +231,9 @@
 Testing
 *******
 
+* HTTP test servers will leak less threads (and sockets) and will not hang on
+  AIX anymore. (Vincent Ladeuil, #405745)
+
 * The way ``bzr selftest --parallel`` generates N partitions of tests to
   run in parallel has changed.  Instead of splitting the list of tests at
   N-1 points, it distributes the tests one-by-one into the partitions in a

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-08-20 09:39:20 +0000
+++ bzrlib/builtins.py	2010-08-31 08:57:28 +0000
@@ -32,7 +32,7 @@
     bzrdir,
     directory_service,
     delta,
-    config,
+    config as _mod_config,
     errors,
     globbing,
     hooks,
@@ -3322,7 +3322,7 @@
                 try:
                     c = Branch.open_containing(u'.')[0].get_config()
                 except errors.NotBranchError:
-                    c = config.GlobalConfig()
+                    c = _mod_config.GlobalConfig()
             else:
                 c = Branch.open(directory).get_config()
             if email:
@@ -3333,7 +3333,7 @@
 
         # display a warning if an email address isn't included in the given name.
         try:
-            config.extract_email_address(name)
+            _mod_config.extract_email_address(name)
         except errors.NoEmailInUsername, e:
             warning('"%s" does not seem to contain an email address.  '
                     'This is allowed, but not recommended.', name)
@@ -3345,7 +3345,7 @@
             else:
                 c = Branch.open(directory).get_config()
         else:
-            c = config.GlobalConfig()
+            c = _mod_config.GlobalConfig()
         c.set_user_option('email', name)
 
 
@@ -3418,13 +3418,13 @@
                 'bzr alias --remove expects an alias to remove.')
         # If alias is not found, print something like:
         # unalias: foo: not found
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.unset_alias(alias_name)
 
     @display_command
     def print_aliases(self):
         """Print out the defined aliases in a similar format to bash."""
-        aliases = config.GlobalConfig().get_aliases()
+        aliases = _mod_config.GlobalConfig().get_aliases()
         for key, value in sorted(aliases.iteritems()):
             self.outf.write('bzr alias %s="%s"\n' % (key, value))
 
@@ -3440,7 +3440,7 @@
 
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the global config."""
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.set_alias(alias_name, alias_command)
 
 
@@ -3570,10 +3570,7 @@
             randomize=None, exclude=None, strict=False,
             load_list=None, debugflag=None, starting_with=None, subunit=False,
             parallel=None, lsprof_tests=False):
-        from bzrlib.tests import selftest
-
-        # Make deprecation warnings visible, unless -Werror is set
-        symbol_versioning.activate_deprecation_warnings(override=False)
+        from bzrlib import tests
 
         if testspecs_list is not None:
             pattern = '|'.join(testspecs_list)
@@ -3620,7 +3617,14 @@
                           "starting_with": starting_with
                           }
         selftest_kwargs.update(self.additional_selftest_args)
-        result = selftest(**selftest_kwargs)
+
+        # Make deprecation warnings visible, unless -Werror is set
+        cleanup = symbol_versioning.activate_deprecation_warnings(
+            override=False)
+        try:
+            result = tests.selftest(**selftest_kwargs)
+        finally:
+            cleanup()
         return int(not result)
 
 
@@ -4807,7 +4811,10 @@
 
 
 class cmd_break_lock(Command):
-    __doc__ = """Break a dead lock on a repository, branch or working directory.
+    __doc__ = """Break a dead lock.
+
+    This command breaks a lock on a repository, branch, working directory or
+    config file.
 
     CAUTION: Locks should only be broken when you are sure that the process
     holding the lock has been stopped.
@@ -4818,17 +4825,27 @@
     :Examples:
         bzr break-lock
         bzr break-lock bzr+ssh://example.com/bzr/foo
+        bzr break-lock --conf ~/.bazaar
     """
+
     takes_args = ['location?']
+    takes_options = [
+        Option('config',
+               help='LOCATION is the directory where the config lock is.'),
+        ]
 
-    def run(self, location=None, show=False):
+    def run(self, location=None, config=False):
         if location is None:
             location = u'.'
-        control, relpath = bzrdir.BzrDir.open_containing(location)
-        try:
-            control.break_lock()
-        except NotImplementedError:
-            pass
+        if config:
+            conf = _mod_config.LockableConfig(file_name=location)
+            conf.break_lock()
+        else:
+            control, relpath = bzrdir.BzrDir.open_containing(location)
+            try:
+                control.break_lock()
+            except NotImplementedError:
+                pass
 
 
 class cmd_wait_until_signalled(Command):

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-07-21 03:14:56 +0000
+++ bzrlib/config.py	2010-08-29 14:32:45 +0000
@@ -65,6 +65,7 @@
 import os
 import sys
 
+from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import errno
@@ -77,11 +78,13 @@
     atomicfile,
     debug,
     errors,
+    lockdir,
     mail_client,
     osutils,
     registry,
     symbol_versioning,
     trace,
+    transport,
     ui,
     urlutils,
     win32utils,
@@ -352,24 +355,79 @@
 class IniBasedConfig(Config):
     """A configuration policy that draws from ini files."""
 
-    def __init__(self, get_filename):
+    def __init__(self, get_filename=symbol_versioning.DEPRECATED_PARAMETER,
+                 file_name=None):
+        """Base class for configuration files using an ini-like syntax.
+
+        :param file_name: The configuration file path.
+        """
         super(IniBasedConfig, self).__init__()
-        self._get_filename = get_filename
+        self.file_name = file_name
+        if symbol_versioning.deprecated_passed(get_filename):
+            symbol_versioning.warn(
+                'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+                ' Use file_name instead.',
+                DeprecationWarning,
+                stacklevel=2)
+            if get_filename is not None:
+                self.file_name = get_filename()
+        else:
+            self.file_name = file_name
+        self._content = None
         self._parser = None
 
-    def _get_parser(self, file=None):
+    @classmethod
+    def from_string(cls, str_or_unicode, file_name=None, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param file_name: The configuration file path.
+
+        :param _save: Whether the file should be saved upon creation.
+        """
+        conf = cls(file_name=file_name)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
+    def _create_from_string(self, str_or_unicode, save):
+        self._content = StringIO(str_or_unicode.encode('utf-8'))
+        # Some tests use in-memory configs, some other always need the config
+        # file to exist on disk.
+        if save:
+            self._write_config_file()
+
+    def _get_parser(self, file=symbol_versioning.DEPRECATED_PARAMETER):
         if self._parser is not None:
             return self._parser
-        if file is None:
-            input = self._get_filename()
+        if symbol_versioning.deprecated_passed(file):
+            symbol_versioning.warn(
+                'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+                ' Use IniBasedConfig(_content=xxx) instead.',
+                DeprecationWarning,
+                stacklevel=2)
+        if self._content is not None:
+            co_input = self._content
+        elif self.file_name is None:
+            raise AssertionError('We have no content to create the config')
         else:
-            input = file
+            co_input = self.file_name
         try:
-            self._parser = ConfigObj(input, encoding='utf-8')
+            self._parser = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        # Make sure self.reload() will use the right file name
+        self._parser.filename = self.file_name
         return self._parser
 
+    def reload(self):
+        """Reload the config file from disk."""
+        if self.file_name is None:
+            raise AssertionError('We need a file name to reload the config')
+        if self._parser is not None:
+            self._parser.reload()
+
     def _get_matching_sections(self):
         """Return an ordered list of (section_name, extra_path) pairs.
 
@@ -479,23 +537,104 @@
         return self.get_user_option('nickname')
 
     def _write_config_file(self):
-        filename = self._get_filename()
-        atomic_file = atomicfile.AtomicFile(filename)
+        if self.file_name is None:
+            raise AssertionError('We cannot save, self.file_name is None')
+        conf_dir = os.path.dirname(self.file_name)
+        ensure_config_dir_exists(conf_dir)
+        atomic_file = atomicfile.AtomicFile(self.file_name)
         self._get_parser().write(atomic_file)
         atomic_file.commit()
         atomic_file.close()
-        osutils.copy_ownership_from_path(filename)
-
-
-class GlobalConfig(IniBasedConfig):
+        osutils.copy_ownership_from_path(self.file_name)
+
+
+class LockableConfig(IniBasedConfig):
+    """A configuration needing explicit locking for access.
+
+    If several processes try to write the config file, the accesses need to be
+    serialized.
+
+    Daughter classes should decorate all methods that update a config with the
+    ``@needs_write_lock`` decorator (they call, directly or indirectly, the
+    ``_write_config_file()`` method. These methods (typically ``set_option()``
+    and variants must reload the config file from disk before calling
+    ``_write_config_file()``), this can be achieved by calling the
+    ``self.reload()`` method. Note that the lock scope should cover both the
+    reading and the writing of the config file which is why the decorator can't
+    be applied to ``_write_config_file()`` only.
+
+    This should be enough to implement the following logic:
+    - lock for exclusive write access,
+    - reload the config file from disk,
+    - set the new value
+    - unlock
+
+    This logic guarantees that a writer can update a value without erasing an
+    update made by another writer.
+    """
+
+    lock_name = 'lock'
+
+    def __init__(self, file_name):
+        super(LockableConfig, self).__init__(file_name=file_name)
+        self.dir = osutils.dirname(osutils.safe_unicode(self.file_name))
+        self.transport = transport.get_transport(self.dir)
+        self._lock = lockdir.LockDir(self.transport, 'lock')
+
+    def _create_from_string(self, unicode_bytes, save):
+        super(LockableConfig, self)._create_from_string(unicode_bytes, False)
+        if save:
+            # We need to handle the saving here (as opposed to IniBasedConfig)
+            # to be able to lock
+            self.lock_write()
+            self._write_config_file()
+            self.unlock()
+
+    def lock_write(self, token=None):
+        """Takes a write lock in the directory containing the config file.
+
+        If the directory doesn't exist it is created.
+        """
+        ensure_config_dir_exists(self.dir)
+        return self._lock.lock_write(token)
+
+    def unlock(self):
+        self._lock.unlock()
+
+    def break_lock(self):
+        self._lock.break_lock()
+
+    def _write_config_file(self):
+        if self._lock is None or not self._lock.is_held:
+            # NB: if the following exception is raised it probably means a
+            # missing @needs_write_lock decorator on one of the callers.
+            raise errors.ObjectNotLocked(self)
+        super(LockableConfig, self)._write_config_file()
+
+
+class GlobalConfig(LockableConfig):
     """The configuration that should be used for a specific location."""
 
+    def __init__(self):
+        super(GlobalConfig, self).__init__(file_name=config_filename())
+
+    @classmethod
+    def from_string(cls, str_or_unicode, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This
+            will be utf-8 encoded.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls()
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
     def get_editor(self):
         return self._get_user_option('editor')
 
-    def __init__(self):
-        super(GlobalConfig, self).__init__(config_filename)
-
+    @needs_write_lock
     def set_user_option(self, option, value):
         """Save option and its value in the configuration."""
         self._set_option(option, value, 'DEFAULT')
@@ -507,12 +646,15 @@
         else:
             return {}
 
+    @needs_write_lock
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the configuration."""
         self._set_option(alias_name, alias_command, 'ALIASES')
 
+    @needs_write_lock
     def unset_alias(self, alias_name):
         """Unset an existing alias."""
+        self.reload()
         aliases = self._get_parser().get('ALIASES')
         if not aliases or alias_name not in aliases:
             raise errors.NoSuchAlias(alias_name)
@@ -520,30 +662,17 @@
         self._write_config_file()
 
     def _set_option(self, option, value, section):
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on bazaar.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         self._get_parser().setdefault(section, {})[option] = value
         self._write_config_file()
 
 
-class LocationConfig(IniBasedConfig):
+class LocationConfig(LockableConfig):
     """A configuration object that gives the policy for a location."""
 
     def __init__(self, location):
-        name_generator = locations_config_filename
-        if (not os.path.exists(name_generator()) and
-                os.path.exists(branches_config_filename())):
-            if sys.platform == 'win32':
-                trace.warning('Please rename %s to %s'
-                              % (branches_config_filename(),
-                                 locations_config_filename()))
-            else:
-                trace.warning('Please rename ~/.bazaar/branches.conf'
-                              ' to ~/.bazaar/locations.conf')
-            name_generator = branches_config_filename
-        super(LocationConfig, self).__init__(name_generator)
+        super(LocationConfig, self).__init__(
+            file_name=locations_config_filename())
         # local file locations are looked up by local path, rather than
         # by file url. This is because the config file is a user
         # file, and we would rather not expose the user to file urls.
@@ -551,6 +680,21 @@
             location = urlutils.local_path_from_url(location)
         self.location = location
 
+    @classmethod
+    def from_string(cls, str_or_unicode, location, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param location: The location url to filter the configuration.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls(location)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
     def _get_matching_sections(self):
         """Return an ordered list of section names matching this location."""
         sections = self._get_parser()
@@ -644,6 +788,7 @@
             if policy_key in self._get_parser()[section]:
                 del self._get_parser()[section][policy_key]
 
+    @needs_write_lock
     def set_user_option(self, option, value, store=STORE_LOCATION):
         """Save option and its value in the configuration."""
         if store not in [STORE_LOCATION,
@@ -651,19 +796,16 @@
                          STORE_LOCATION_APPENDPATH]:
             raise ValueError('bad storage policy %r for %r' %
                 (store, option))
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on locations.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         location = self.location
         if location.endswith('/'):
             location = location[:-1]
-        if (not location in self._get_parser() and
-            not location + '/' in self._get_parser()):
-            self._get_parser()[location]={}
-        elif location + '/' in self._get_parser():
+        parser = self._get_parser()
+        if not location in parser and not location + '/' in parser:
+            parser[location] = {}
+        elif location + '/' in parser:
             location = location + '/'
-        self._get_parser()[location][option]=value
+        parser[location][option]=value
         # the allowed values of store match the config policies
         self._set_option_policy(location, option, store)
         self._write_config_file()
@@ -672,6 +814,16 @@
 class BranchConfig(Config):
     """A configuration object giving the policy for a branch."""
 
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self._branch_data_config = None
+        self._global_config = None
+        self.branch = branch
+        self.option_sources = (self._get_location_config,
+                               self._get_branch_data_config,
+                               self._get_global_config)
+
     def _get_branch_data_config(self):
         if self._branch_data_config is None:
             self._branch_data_config = TreeConfig(self.branch)
@@ -775,16 +927,6 @@
         """See Config.gpg_signing_command."""
         return self._get_safe_value('_gpg_signing_command')
 
-    def __init__(self, branch):
-        super(BranchConfig, self).__init__()
-        self._location_config = None
-        self._branch_data_config = None
-        self._global_config = None
-        self.branch = branch
-        self.option_sources = (self._get_location_config,
-                               self._get_branch_data_config,
-                               self._get_global_config)
-
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_safe_value('_post_commit')
@@ -854,11 +996,6 @@
     return osutils.pathjoin(config_dir(), 'bazaar.conf')
 
 
-def branches_config_filename():
-    """Return per-user configuration ini file filename."""
-    return osutils.pathjoin(config_dir(), 'branches.conf')
-
-
 def locations_config_filename():
     """Return per-user configuration ini file filename."""
     return osutils.pathjoin(config_dir(), 'locations.conf')

=== modified file 'bzrlib/doc_generate/autodoc_man.py'
--- bzrlib/doc_generate/autodoc_man.py	2010-06-02 04:50:52 +0000
+++ bzrlib/doc_generate/autodoc_man.py	2010-08-28 04:21:40 +0000
@@ -241,7 +241,7 @@
 .br
 log10 = log --short -r -10..-1
 .SH "SEE ALSO"
-.UR http://www.bazaar.canonical.com/
-.BR http://www.bazaar.canonical.com/
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/
 """
 

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2009-09-20 22:12:36 +0000
+++ bzrlib/hashcache.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-06-02 04:50:35 +0000
+++ bzrlib/help_topics/__init__.py	2010-08-28 04:21:40 +0000
@@ -279,7 +279,7 @@
 
 _basic_help = \
 """Bazaar %s -- a free distributed version-control tool
-http://www.bazaar.canonical.com/
+http://bazaar.canonical.com/
 
 Basic commands:
   bzr init           makes this directory a versioned branch

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-07-09 16:16:11 +0000
+++ bzrlib/osutils.py	2010-08-24 12:36:53 +0000
@@ -2065,6 +2065,29 @@
             report_activity(sent, 'write')
 
 
+def connect_socket(address):
+    # Slight variation of the socket.create_connection() function (provided by
+    # python-2.6) that can fail if getaddrinfo returns an empty list. We also
+    # provide it for previous python versions. Also, we don't use the timeout
+    # parameter (provided by the python implementation) so we don't implement
+    # it either).
+    err = socket.error('getaddrinfo returns an empty list')
+    host, port = address
+    for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
+        af, socktype, proto, canonname, sa = res
+        sock = None
+        try:
+            sock = socket.socket(af, socktype, proto)
+            sock.connect(sa)
+            return sock
+
+        except socket.error, err:
+            # 'err' is now the most recent error
+            if sock is not None:
+                sock.close()
+    raise err
+
+
 def dereference_path(path):
     """Determine the real path to a file.
 

=== modified file 'bzrlib/plugins/launchpad/test_account.py'
--- bzrlib/plugins/launchpad/test_account.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/launchpad/test_account.py	2010-08-30 07:34:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@
 class LaunchpadAccountTests(TestCaseInTempDir):
 
     def setup_config(self, text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return my_config
 
     def test_get_lp_login_unconfigured(self):

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2010-03-05 07:27:58 +0000
+++ bzrlib/smart/server.py	2010-07-01 15:25:41 +0000
@@ -18,6 +18,7 @@
 
 import errno
 import os.path
+import select
 import socket
 import sys
 import threading
@@ -51,18 +52,24 @@
     hooks: An instance of SmartServerHooks.
     """
 
-    def __init__(self, backing_transport, host='127.0.0.1', port=0,
-                 root_client_path='/'):
+    def __init__(self, backing_transport, root_client_path='/'):
         """Construct a new server.
 
         To actually start it running, call either start_background_thread or
         serve.
 
         :param backing_transport: The transport to serve.
+        :param root_client_path: The client path that will correspond to root
+            of backing_transport.
+        """
+        self.backing_transport = backing_transport
+        self.root_client_path = root_client_path
+
+    def start_server(self, host, port):
+        """Create the server listening socket.
+
         :param host: Name of the interface to listen on.
         :param port: TCP port to listen on, or 0 to allocate a transient port.
-        :param root_client_path: The client path that will correspond to root
-            of backing_transport.
         """
         # let connections timeout so that we get a chance to terminate
         # Keep a reference to the exceptions we want to catch because the socket
@@ -89,15 +96,10 @@
         self.port = self._sockname[1]
         self._server_socket.listen(1)
         self._server_socket.settimeout(1)
-        self.backing_transport = backing_transport
         self._started = threading.Event()
         self._stopped = threading.Event()
-        self.root_client_path = root_client_path
 
-    def serve(self, thread_name_suffix=''):
-        self._should_terminate = False
-        # for hooks we are letting code know that a server has started (and
-        # later stopped).
+    def _backing_urls(self):
         # There are three interesting urls:
         # The URL the server can be contacted on. (e.g. bzr://host/)
         # The URL that a commit done on the same machine as the server will
@@ -113,15 +115,32 @@
         # The latter two urls are different aliases to the servers url,
         # so we group those in a list - as there might be more aliases
         # in the future.
-        backing_urls = [self.backing_transport.base]
+        urls = [self.backing_transport.base]
         try:
-            backing_urls.append(self.backing_transport.external_url())
+            urls.append(self.backing_transport.external_url())
         except errors.InProcessTransport:
             pass
+        return urls
+
+    def run_server_started_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
         for hook in SmartTCPServer.hooks['server_started']:
             hook(backing_urls, self.get_url())
         for hook in SmartTCPServer.hooks['server_started_ex']:
             hook(backing_urls, self)
+
+    def run_server_stopped_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
+        for hook in SmartTCPServer.hooks['server_stopped']:
+            hook(backing_urls, self.get_url())
+
+    def serve(self, thread_name_suffix=''):
+        self._should_terminate = False
+        # for hooks we are letting code know that a server has started (and
+        # later stopped).
+        self.run_server_started_hooks()
         self._started.set()
         try:
             try:
@@ -155,8 +174,7 @@
             except self._socket_error:
                 # ignore errors on close
                 pass
-            for hook in SmartTCPServer.hooks['server_stopped']:
-                hook(backing_urls, self.get_url())
+            self.run_server_stopped_hooks()
 
     def get_url(self):
         """Return the url of the server"""
@@ -172,8 +190,12 @@
         thread_name = 'smart-server-child' + thread_name_suffix
         connection_thread = threading.Thread(
             None, handler.serve, name=thread_name)
+        # FIXME: This thread is never joined, it should at least be collected
+        # somewhere so that tests that want to check for leaked threads can get
+        # rid of them -- vila 20100531
         connection_thread.setDaemon(True)
         connection_thread.start()
+        return connection_thread
 
     def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
@@ -324,7 +346,8 @@
                 host = medium.BZR_DEFAULT_INTERFACE
             if port is None:
                 port = medium.BZR_DEFAULT_PORT
-            smart_server = SmartTCPServer(self.transport, host=host, port=port)
+            smart_server = SmartTCPServer(self.transport)
+            smart_server.start_server(host, port)
             trace.note('listening on port: %s' % smart_server.port)
         self.smart_server = smart_server
 

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2010-05-25 16:45:12 +0000
+++ bzrlib/store/text.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2010-06-29 16:21:13 +0000
+++ bzrlib/symbol_versioning.py	2010-08-28 15:34:24 +0000
@@ -29,6 +29,9 @@
            'warn',
            ]
 
+
+import warnings
+# Import the 'warn' symbol so bzrlib can call it even if we redefine it
 from warnings import warn
 
 import bzrlib
@@ -296,7 +299,6 @@
     :param error_only: Only match an 'error' filter
     :return: True if a filter is found, False otherwise
     """
-    import warnings
     for filter in warnings.filters:
         if issubclass(DeprecationWarning, filter[2]):
             # This filter will effect DeprecationWarning
@@ -305,6 +307,19 @@
     return False
 
 
+def _remove_filter_callable(filter):
+    """Build and returns a callable removing filter from the warnings.
+
+    :param filter: The filter to remove (can be None).
+
+    :return: A callable that will remove filter from warnings.filters.
+    """
+    def cleanup():
+        if filter:
+            warnings.filters.remove(filter)
+    return cleanup
+
+
 def suppress_deprecation_warnings(override=True):
     """Call this function to suppress all deprecation warnings.
 
@@ -314,18 +329,17 @@
 
     :param override: If True, always set the ignore, if False, only set the
         ignore if there isn't already a filter.
+
     :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=False):
         # If there is already a filter effecting suppress_deprecation_warnings,
         # then skip it.
-        return
-    warnings.filterwarnings('ignore', category=DeprecationWarning)
-    filter = warnings.filters[0]
-    def cleanup():
-        warnings.filters.remove(filter)
-    return cleanup
+        filter = None
+    else:
+        warnings.filterwarnings('ignore', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)
 
 
 def activate_deprecation_warnings(override=True):
@@ -342,10 +356,14 @@
     :param override: If False, only add a filter if there isn't an error filter
         already. (This slightly differs from suppress_deprecation_warnings, in
         because it always overrides everything but -Werror).
+
+    :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=True):
         # DeprecationWarnings are already turned into errors, don't downgrade
         # them to 'default'.
-        return
-    warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = None
+    else:
+        warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)

=== modified file 'bzrlib/tests/EncodingAdapter.py'
--- bzrlib/tests/EncodingAdapter.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/EncodingAdapter.py	2010-05-24 01:05:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,9 +19,6 @@
 
 from copy import deepcopy
 
-from bzrlib.tests import TestSuite
-
-
 # prefix for micro (1/1000000)
 _mu = u'\xb5'
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2010-08-24 23:20:14 +0000
+++ bzrlib/tests/__init__.py	2010-08-30 07:42:12 +0000
@@ -114,11 +114,6 @@
     TestUtil,
     treeshape,
     )
-from bzrlib.tests.http_server import HttpServer
-from bzrlib.tests.TestUtil import (
-                          TestSuite,
-                          TestLoader,
-                          )
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
 import bzrlib.version_info_formats.format_custom
@@ -848,6 +843,8 @@
         # going away but leak one) but it seems less likely than the actual
         # false positives (the test see threads going away and does not leak).
         if leaked_threads > 0:
+            if 'threads' in selftest_debug_flags:
+                print '%s is leaking, active is now %d' % (self.id(), active)
             TestCase._leaking_threads_tests += 1
             if TestCase._first_thread_leaker_id is None:
                 TestCase._first_thread_leaker_id = self.id()
@@ -2439,6 +2436,15 @@
 
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
+        # Ensure that ConnectedTransport doesn't leak sockets
+        def get_transport_with_cleanup(*args, **kwargs):
+            t = orig_get_transport(*args, **kwargs)
+            if isinstance(t, _mod_transport.ConnectedTransport):
+                self.addCleanup(t.disconnect)
+            return t
+
+        orig_get_transport = self.overrideAttr(_mod_transport, 'get_transport',
+                                               get_transport_with_cleanup)
         self._make_test_root()
         self.addCleanup(os.chdir, os.getcwdu())
         self.makeAndChdirToTestDir()
@@ -2736,9 +2742,10 @@
     """
 
     def setUp(self):
+        from bzrlib.tests import http_server
         super(ChrootedTestCase, self).setUp()
         if not self.vfs_transport_factory == memory.MemoryServer:
-            self.transport_readonly_server = HttpServer
+            self.transport_readonly_server = http_server.HttpServer
 
 
 def condition_id_re(pattern):
@@ -3065,7 +3072,7 @@
     return suite
 
 
-class TestDecorator(TestSuite):
+class TestDecorator(TestUtil.TestSuite):
     """A decorator for TestCase/TestSuite objects.
     
     Usually, subclasses should override __iter__(used when flattening test
@@ -3074,7 +3081,7 @@
     """
 
     def __init__(self, suite):
-        TestSuite.__init__(self)
+        TestUtil.TestSuite.__init__(self)
         self.addTest(suite)
 
     def countTestCases(self):
@@ -3248,7 +3255,7 @@
 
     test_blocks = partition_tests(suite, concurrency)
     for process_tests in test_blocks:
-        process_suite = TestSuite()
+        process_suite = TestUtil.TestSuite()
         process_suite.addTests(process_tests)
         c2pread, c2pwrite = os.pipe()
         pid = os.fork()
@@ -3409,6 +3416,8 @@
 #                           rather than failing tests. And no longer raise
 #                           LockContention when fctnl locks are not being used
 #                           with proper exclusion rules.
+#   -Ethreads               Will display thread ident at creation/join time to
+#                           help track thread leaks
 selftest_debug_flags = set()
 
 
@@ -3808,6 +3817,7 @@
         'bzrlib.tests.test_switch',
         'bzrlib.tests.test_symbol_versioning',
         'bzrlib.tests.test_tag',
+        'bzrlib.tests.test_test_server',
         'bzrlib.tests.test_testament',
         'bzrlib.tests.test_textfile',
         'bzrlib.tests.test_textmerge',
@@ -4004,7 +4014,7 @@
     ...     bzrlib.tests.test_sampler.DemoTest('test_nothing'),
     ...     [('one', dict(param=1)),
     ...      ('two', dict(param=2))],
-    ...     TestSuite())
+    ...     TestUtil.TestSuite())
     >>> tests = list(iter_suite_tests(r))
     >>> len(tests)
     2

=== modified file 'bzrlib/tests/blackbox/test_alias.py'
--- bzrlib/tests/blackbox/test_alias.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_alias.py	2010-08-29 14:32:45 +0000
@@ -16,14 +16,14 @@
 #
 
 """Tests of the 'bzr alias' command."""
-import os
-import codecs
-
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
-
-
-class TestAlias(TestCaseWithTransport):
+
+from bzrlib import (
+    config,
+    tests,
+    )
+
+
+class TestAlias(tests.TestCaseWithTransport):
 
     def test_list_alias_with_none(self):
         """Calling alias with no parameters lists existing aliases."""
@@ -46,7 +46,6 @@
 
     def test_unicode_alias(self):
         """Unicode aliases should work (Bug #529930)"""
-        config_enc = 'utf-8'
         file_name = u'foo\xb6'
 
         tree = self.make_branch_and_tree('.')
@@ -54,11 +53,8 @@
         tree.add(file_name)
         tree.commit('added')
 
-        ensure_config_dir_exists()
-        CONFIG=(u'[ALIASES]\n'
-                u'ust=st foo\xb6\n')
-
-        codecs.open(config_filename(),'w', config_enc).write(CONFIG)
+        config.GlobalConfig.from_string(
+            u'[ALIASES]\nust=st %s\n' % (file_name,), save=True)
 
         out, err = self.run_bzr('ust')
         self.assertEquals(err, '')

=== modified file 'bzrlib/tests/blackbox/test_aliases.py'
--- bzrlib/tests/blackbox/test_aliases.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_aliases.py	2010-08-29 14:32:45 +0000
@@ -18,10 +18,10 @@
 
 import os
 
+from bzrlib import config
 from bzrlib.branch import Branch
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
 class TestAliases(TestCaseWithTransport):
@@ -35,18 +35,11 @@
             return self.run_bzr(args, **kwargs)[1]
 
 
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-                "c=cat\n"
-                "c1=cat -r 1\n"
-                "c2=cat -r 1 -r2\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+c1=cat -r 1
+c2=cat -r 1 -r2
+''', save=True)
 
         str1 = 'foo\n'
         str2 = 'bar\n'

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2010-07-21 13:32:27 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2010-08-23 16:34:38 +0000
@@ -22,8 +22,10 @@
 from bzrlib import (
     branch,
     bzrdir,
+    config,
     errors,
     lockdir,
+    osutils,
     tests,
     )
 
@@ -73,7 +75,7 @@
         # however, we dont test breaking the working tree because we
         # cannot accurately do so right now: the dirstate lock is held
         # by an os lock, and we need to spawn a separate process to lock it
-        # thne kill -9 it.
+        # then kill -9 it.
         # sketch of test:
         # lock most of the dir:
         self.wt.branch.lock_write()
@@ -101,3 +103,24 @@
         out, err = self.run_bzr('break-lock foo')
         self.assertEqual('', out)
         self.assertEqual('', err)
+
+class TestConfigBreakLock(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigBreakLock, self).setUp()
+        self.config_file_name = './my.conf'
+        self.build_tree_contents([(self.config_file_name,
+                                   '[DEFAULT]\none=1\n')])
+        self.config = config.LockableConfig(file_name=self.config_file_name)
+        self.config.lock_write()
+
+    def test_create_pending_lock(self):
+        self.addCleanup(self.config.unlock)
+        self.assertTrue(self.config._lock.is_held)
+
+    def test_break_lock(self):
+        self.run_bzr('break-lock --config %s'
+                     % osutils.dirname(self.config_file_name),
+                     stdin="y\n")
+        self.assertRaises(errors.LockBroken, self.config.unlock)
+

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_help.py	2010-08-29 14:32:45 +0000
@@ -20,8 +20,8 @@
 
 
 import bzrlib
+from bzrlib import config
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
 class TestHelp(TestCaseWithTransport):
@@ -161,12 +161,10 @@
     def test_help_with_aliases(self):
         original = self.run_bzr('help cat')[0]
 
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-        "c=cat\n"
-        "cat=cat\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+cat=cat
+''', save=True)
 
         expected = original + "'bzr cat' is an alias for 'bzr cat'.\n"
         self.assertEqual(expected, self.run_bzr('help cat')[0])

=== modified file 'bzrlib/tests/ftp_server/medusa_based.py'
--- bzrlib/tests/ftp_server/medusa_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/medusa_based.py	2010-06-30 15:19:36 +0000
@@ -254,6 +254,9 @@
         self._async_thread = threading.Thread(
                 target=FTPTestServer._asyncore_loop_ignore_EBADF,
                 kwargs={'timeout':0.1, 'count':10000})
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._async_thread.ident,))
         self._async_thread.setDaemon(True)
         self._async_thread.start()
 
@@ -261,6 +264,9 @@
         self._ftp_server.close()
         asyncore.close_all()
         self._async_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._async_thread.ident,))
 
     @staticmethod
     def _asyncore_loop_ignore_EBADF(*args, **kwargs):

=== modified file 'bzrlib/tests/ftp_server/pyftpdlib_based.py'
--- bzrlib/tests/ftp_server/pyftpdlib_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/pyftpdlib_based.py	2010-06-30 15:19:36 +0000
@@ -28,6 +28,7 @@
 
 from bzrlib import (
     osutils,
+    tests,
     trace,
     )
 from bzrlib.tests import test_server
@@ -183,6 +184,9 @@
         self._ftpd_starting.acquire() # So it can be released by the server
         self._ftpd_thread = threading.Thread(target=self._run_server,)
         self._ftpd_thread.start()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._ftpd_thread.ident,))
         # Wait for the server thread to start (i.e release the lock)
         self._ftpd_starting.acquire()
         self._ftpd_starting.release()
@@ -195,6 +199,9 @@
         self._ftp_server.close()
         self._ftpd_running = False
         self._ftpd_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._ftpd_thread.ident,))
 
     def _run_server(self):
         """Run the server until stop_server is called, shut it down properly then.

=== modified file 'bzrlib/tests/http_server.py'
--- bzrlib/tests/http_server.py	2010-07-29 11:17:57 +0000
+++ bzrlib/tests/http_server.py	2010-08-24 13:01:06 +0000
@@ -30,7 +30,11 @@
 import urllib
 import urlparse
 
-from bzrlib import transport
+from bzrlib import (
+    osutils,
+    tests,
+    transport,
+    )
 from bzrlib.tests import test_server
 from bzrlib.transport import local
 
@@ -71,6 +75,14 @@
                 self.headers.get('referer', '-'),
                 self.headers.get('user-agent', '-'))
 
+    def handle(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle(self)
+        # Some client (pycurl, I'm looking at you) are more picky than others
+        # and require that the socket itself is closed
+        # (SocketServer.StreamRequestHandler only close the two associated
+        # 'makefile' objects)
+        self.connection.close()
+
     def handle_one_request(self):
         """Handle a single HTTP request.
 
@@ -78,7 +90,7 @@
         connection early to avoid polluting the test results.
         """
         try:
-            SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+            self._handle_one_request()
         except socket.error, e:
             # Any socket error should close the connection, but some errors are
             # due to the client closing early and we don't want to pollute test
@@ -124,6 +136,9 @@
         if self.command != 'HEAD' and code >= 200 and code not in (204, 304):
             self.wfile.write(content)
 
+    def _handle_one_request(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+
     _range_regexp = re.compile(r'^(?P<start>\d+)-(?P<end>\d+)$')
     _tail_regexp = re.compile(r'^-(?P<tail>\d+)$')
 
@@ -248,12 +263,12 @@
             # mode may cause newline translations, making the
             # actual size of the content transmitted *less* than
             # the content-length!
-            file = open(path, 'rb')
+            f = open(path, 'rb')
         except IOError:
             self.send_error(404, "File not found")
             return
 
-        file_size = os.fstat(file.fileno())[6]
+        file_size = os.fstat(f.fileno())[6]
         tail, ranges = self.parse_ranges(ranges_header_value)
         # Normalize tail into ranges
         if tail != 0:
@@ -280,7 +295,7 @@
             # RFC2616 14.16 and 14.35 says that when a server
             # encounters unsatisfiable range specifiers, it
             # SHOULD return a 416.
-            file.close()
+            f.close()
             # FIXME: We SHOULD send a Content-Range header too,
             # but the implementation of send_error does not
             # allows that. So far.
@@ -289,10 +304,10 @@
 
         if len(ranges) == 1:
             (start, end) = ranges[0]
-            self.get_single_range(file, file_size, start, end)
+            self.get_single_range(f, file_size, start, end)
         else:
-            self.get_multiple_ranges(file, file_size, ranges)
-        file.close()
+            self.get_multiple_ranges(f, file_size, ranges)
+        f.close()
 
     def translate_path(self, path):
         """Translate a /-separated PATH to the local filename syntax.
@@ -354,52 +369,17 @@
         self.test_case_server = test_case_server
         self._home_dir = test_case_server._home_dir
 
-    def stop_server(self):
-         """Called to clean-up the server.
-
-         Since the server may be (surely is, even) in a blocking listen, we
-         shutdown its socket before closing it.
-         """
-         # Note that is this executed as part of the implicit tear down in the
-         # main thread while the server runs in its own thread. The clean way
-         # to tear down the server is to instruct him to stop accepting
-         # connections and wait for the current connection(s) to end
-         # naturally. To end the connection naturally, the http transports
-         # should close their socket when they do not need to talk to the
-         # server anymore. This happens naturally during the garbage collection
-         # phase of the test transport objetcs (the server clients), so we
-         # don't have to worry about them.  So, for the server, we must tear
-         # down here, from the main thread, when the test have ended.  Note
-         # that since the server is in a blocking operation and since python
-         # use select internally, shutting down the socket is reliable and
-         # relatively clean.
-         try:
-             self.socket.shutdown(socket.SHUT_RDWR)
-         except socket.error, e:
-             # WSAENOTCONN (10057) 'Socket is not connected' is harmless on
-             # windows (occurs before the first connection attempt
-             # vila--20071230)
-
-             # 'Socket is not connected' can also occur on OSX, with a
-             # "regular" ENOTCONN (when something went wrong during test case
-             # setup leading to self.setUp() *not* being called but
-             # self.stop_server() still being called -- vila20081106
-             if not len(e.args) or e.args[0] not in (errno.ENOTCONN, 10057):
-                 raise
-         # Let the server properly close the socket
-         self.server_close()
-
-
-class TestingHTTPServer(SocketServer.TCPServer, TestingHTTPServerMixin):
+
+class TestingHTTPServer(test_server.TestingTCPServer, TestingHTTPServerMixin):
 
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingTCPServer.__init__(self, server_address,
+                                              request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.TCPServer.__init__(self, server_address,
-                                        request_handler_class)
-
-
-class TestingThreadingHTTPServer(SocketServer.ThreadingTCPServer,
+
+
+class TestingThreadingHTTPServer(test_server.TestingThreadingTCPServer,
                                  TestingHTTPServerMixin):
     """A threading HTTP test server for HTTP 1.1.
 
@@ -407,36 +387,14 @@
     server, we need an independent connection for each of them. We achieve that
     by spawning a new thread for each connection.
     """
-
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(self, server_address,
+                                                       request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.ThreadingTCPServer.__init__(self, server_address,
-                                                 request_handler_class)
-        # Decides how threads will act upon termination of the main
-        # process. This is prophylactic as we should not leave the threads
-        # lying around.
-        self.daemon_threads = True
-
-    def process_request_thread(self, request, client_address):
-        SocketServer.ThreadingTCPServer.process_request_thread(
-            self, request, client_address)
-        # Under some circumstances (as in bug #383920), we need to force the
-        # shutdown as python delays it until gc occur otherwise and the client
-        # may hang.
-        try:
-            # The request process has been completed, the thread is about to
-            # die, let's shutdown the socket if we can.
-            request.shutdown(socket.SHUT_RDWR)
-        except (socket.error, select.error), e:
-            if e[0] in (errno.EBADF, errno.ENOTCONN):
-                # Right, the socket is already down
-                pass
-            else:
-                raise
-
-
-class HttpServer(transport.Server):
+
+
+class HttpServer(test_server.TestingTCPServerInAThread):
     """A test server for http transports.
 
     Subclasses can provide a specific request handler.
@@ -464,82 +422,33 @@
         :param protocol_version: if specified, will override the protocol
             version of the request handler.
         """
-        transport.Server.__init__(self)
-        self.request_handler = request_handler
+        # Depending on the protocol version, we will create the approriate
+        # server
+        if protocol_version is None:
+            # Use the request handler one
+            proto_vers = request_handler.protocol_version
+        else:
+            # Use our own, it will be used to override the request handler
+            # one too.
+            proto_vers = protocol_version
+        # Get the appropriate server class for the required protocol
+        serv_cls = self.http_server_class.get(proto_vers, None)
+        if serv_cls is None:
+            raise httplib.UnknownProtocol(proto_vers)
         self.host = 'localhost'
         self.port = 0
-        self._httpd = None
-        self.protocol_version = protocol_version
+        super(HttpServer, self).__init__((self.host, self.port),
+                                         serv_cls,
+                                         request_handler)
+        self.protocol_version = proto_vers
         # Allows tests to verify number of GET requests issued
         self.GET_request_nb = 0
-
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler, self)
-
-    def __repr__(self):
-        return "%s(%s:%s)" % \
-            (self.__class__.__name__, self.host, self.port)
-
-    def _get_httpd(self):
-        if self._httpd is None:
-            rhandler = self.request_handler
-            # Depending on the protocol version, we will create the approriate
-            # server
-            if self.protocol_version is None:
-                # Use the request handler one
-                proto_vers = rhandler.protocol_version
-            else:
-                # Use our own, it will be used to override the request handler
-                # one too.
-                proto_vers = self.protocol_version
-            # Create the appropriate server for the required protocol
-            serv_cls = self.http_server_class.get(proto_vers, None)
-            if serv_cls is None:
-                raise httplib.UnknownProtocol(proto_vers)
-            else:
-                self._httpd = self.create_httpd(serv_cls, rhandler)
-            self.host, self.port = self._httpd.socket.getsockname()
-        return self._httpd
-
-    def _http_start(self):
-        """Server thread main entry point. """
-        self._http_running = False
-        try:
-            try:
-                httpd = self._get_httpd()
-                self._http_base_url = '%s://%s:%s/' % (self._url_protocol,
-                                                       self.host, self.port)
-                self._http_running = True
-            except:
-                # Whatever goes wrong, we save the exception for the main
-                # thread. Note that since we are running in a thread, no signal
-                # can be received, so we don't care about KeyboardInterrupt.
-                self._http_exception = sys.exc_info()
-        finally:
-            # Release the lock or the main thread will block and the whole
-            # process will hang.
-            self._http_starting.release()
-
-        # From now on, exceptions are taken care of by the
-        # SocketServer.BaseServer or the request handler.
-        while self._http_running:
-            try:
-                # Really an HTTP connection but the python framework is generic
-                # and call them requests
-                httpd.handle_request()
-            except socket.timeout:
-                pass
-            except (socket.error, select.error), e:
-                if (e[0] == errno.EBADF
-                    or (sys.platform == 'win32' and e[0] == 10038)):
-                    # Starting with python-2.6, handle_request may raise socket
-                    # or select exceptions when the server is shut down (as we
-                    # do).
-                    # 10038 = WSAENOTSOCK
-                    # http://msdn.microsoft.com/en-us/library/ms740668%28VS.85%29.aspx
-                    pass
-                else:
-                    raise
+        self._http_base_url = None
+        self.logs = []
+
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self)
 
     def _get_remote_url(self, path):
         path_parts = path.split(os.path.sep)
@@ -570,36 +479,15 @@
                 or isinstance(backing_transport_server,
                               test_server.LocalURLServer)):
             raise AssertionError(
-                "HTTPServer currently assumes local transport, got %s" % \
+                "HTTPServer currently assumes local transport, got %s" %
                 backing_transport_server)
         self._home_dir = os.getcwdu()
         self._local_path_parts = self._home_dir.split(os.path.sep)
-        self._http_base_url = None
-
-        # Create the server thread
-        self._http_starting = threading.Lock()
-        self._http_starting.acquire()
-        self._http_thread = threading.Thread(target=self._http_start)
-        self._http_thread.setDaemon(True)
-        self._http_exception = None
-        self._http_thread.start()
-
-        # Wait for the server thread to start (i.e release the lock)
-        self._http_starting.acquire()
-
-        if self._http_exception is not None:
-            # Something went wrong during server start
-            exc_class, exc_value, exc_tb = self._http_exception
-            raise exc_class, exc_value, exc_tb
-        self._http_starting.release()
         self.logs = []
 
-    def stop_server(self):
-        self._httpd.stop_server()
-        self._http_running = False
-        # We don't need to 'self._http_thread.join()' here since the thread is
-        # a daemonic one and will be garbage collected anyway. Joining just
-        # slows us down for no added benefit.
+        super(HttpServer, self).start_server()
+        self._http_base_url = '%s://%s:%s/' % (
+            self._url_protocol, self.host, self.port)
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""

=== modified file 'bzrlib/tests/http_utils.py'
--- bzrlib/tests/http_utils.py	2010-06-11 10:59:23 +0000
+++ bzrlib/tests/http_utils.py	2010-06-23 16:43:16 +0000
@@ -28,13 +28,14 @@
     errors,
     osutils,
     tests,
-    )
-from bzrlib.smart import medium, protocol
+    transport,
+    )
+from bzrlib.smart import (
+    medium,
+    protocol,
+    )
 from bzrlib.tests import http_server
-from bzrlib.transport import (
-    chroot,
-    get_transport,
-    )
+from bzrlib.transport import chroot
 
 
 class HTTPServerWithSmarts(http_server.HttpServer):
@@ -56,11 +57,12 @@
 
     def do_POST(self):
         """Hand the request off to a smart server instance."""
-        backing = get_transport(self.server.test_case_server._home_dir)
+        backing = transport.get_transport(
+            self.server.test_case_server._home_dir)
         chroot_server = chroot.ChrootServer(backing)
         chroot_server.start_server()
         try:
-            t = get_transport(chroot_server.get_url())
+            t = transport.get_transport(chroot_server.get_url())
             self.do_POST_inner(t)
         finally:
             chroot_server.stop_server()
@@ -107,18 +109,21 @@
     backed by regular disk files.
     """
 
-    # This can be overriden or parametrized by daughter clasess if needed, but
-    # it must exist so that the create_transport_readonly_server() method can
-    # propagate it.
+    # These attributes can be overriden or parametrized by daughter clasess if
+    # needed, but must exist so that the create_transport_readonly_server()
+    # method (or any method creating an http(s) server) can propagate it.
     _protocol_version = None
+    _url_protocol = 'http'
 
     def setUp(self):
         super(TestCaseWithWebserver, self).setUp()
         self.transport_readonly_server = http_server.HttpServer
 
     def create_transport_readonly_server(self):
-        return self.transport_readonly_server(
+        server = self.transport_readonly_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
 
 class TestCaseWithTwoWebservers(TestCaseWithWebserver):
@@ -137,8 +142,10 @@
 
         This is mostly a hook for daughter classes.
         """
-        return self.transport_secondary_server(
+        server = self.transport_secondary_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def get_secondary_server(self):
         """Get the server instance for the secondary transport."""
@@ -147,6 +154,15 @@
             self.start_server(self.__secondary_server)
         return self.__secondary_server
 
+    def get_secondary_url(self, relpath=None):
+        base = self.get_secondary_server().get_url()
+        return self._adjust_url(base, relpath)
+
+    def get_secondary_transport(self, relpath=None):
+        t = transport.get_transport(self.get_secondary_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
 
 class ProxyServer(http_server.HttpServer):
     """A proxy test server for http transports."""
@@ -226,20 +242,39 @@
    The 'old' server is redirected to the 'new' server.
    """
 
+   def setUp(self):
+       super(TestCaseWithRedirectedWebserver, self).setUp()
+       # The redirections will point to the new server
+       self.new_server = self.get_readonly_server()
+       # The requests to the old server will be redirected to the new server
+       self.old_server = self.get_secondary_server()
+
    def create_transport_secondary_server(self):
        """Create the secondary server redirecting to the primary server"""
        new = self.get_readonly_server()
        redirecting = HTTPServerRedirecting(
            protocol_version=self._protocol_version)
        redirecting.redirect_to(new.host, new.port)
+       redirecting._url_protocol = self._url_protocol
        return redirecting
 
-   def setUp(self):
-       super(TestCaseWithRedirectedWebserver, self).setUp()
-       # The redirections will point to the new server
-       self.new_server = self.get_readonly_server()
-       # The requests to the old server will be redirected
-       self.old_server = self.get_secondary_server()
+   def get_old_url(self, relpath=None):
+        base = self.old_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_old_transport(self, relpath=None):
+        t = transport.get_transport(self.get_old_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
+   def get_new_url(self, relpath=None):
+        base = self.new_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_new_transport(self, relpath=None):
+        t = transport.get_transport(self.get_new_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
 
 
 class AuthRequestHandler(http_server.TestingHTTPRequestHandler):

=== modified file 'bzrlib/tests/https_server.py'
--- bzrlib/tests/https_server.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/https_server.py	2010-07-01 06:40:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 from bzrlib.tests import (
     http_server,
     ssl_certs,
+    test_server,
     )
 
 
@@ -30,17 +31,25 @@
         self.key_file = key_file
         self.cert_file = cert_file
 
-    def get_request (self):
-        """Get the request and client address from the socket.
-
-        This is called in response to a connection issued to the server, we
-        wrap the socket with SSL.
+    def _get_ssl_request (self, sock, addr):
+        """Wrap the socket with SSL"""
+        ssl_sock = ssl.wrap_socket(sock, server_side=True,
+                                   keyfile=self.key_file,
+                                   certfile=self.cert_file,
+                                   do_handshake_on_connect=False)
+        return ssl_sock, addr
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket !
         """
-        sock, addr = self.socket.accept()
-        sslconn = ssl.wrap_socket(sock, server_side=True,
-                                  keyfile=self.key_file,
-                                  certfile=self.cert_file)
-        return sslconn, addr
+        serving = test_server.TestingTCPServerMixin.verify_request(
+            self, request, client_address)
+        if serving:
+            request.do_handshake()
+        return serving
 
 
 class TestingHTTPSServer(TestingHTTPSServerMixin,
@@ -52,6 +61,10 @@
         http_server.TestingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class TestingThreadingHTTPSServer(TestingHTTPSServerMixin,
                                   http_server.TestingThreadingHTTPServer):
@@ -62,6 +75,10 @@
         http_server.TestingThreadingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingThreadingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class HTTPSServer(http_server.HttpServer):
 
@@ -73,7 +90,7 @@
                          }
 
     # Provides usable defaults since an https server requires both a
-    # private key and certificate to work.
+    # private key and a certificate to work.
     def __init__(self, request_handler=http_server.TestingHTTPRequestHandler,
                  protocol_version=None,
                  key_file=ssl_certs.build_path('server_without_pass.key'),
@@ -84,9 +101,10 @@
         self.cert_file = cert_file
         self.temp_files = []
 
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler,
-                        self, self.key_file, self.cert_file)
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self,
+            self.key_file, self.cert_file)
 
 
 class HTTPSServer_urllib(HTTPSServer):

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2010-04-29 08:59:13 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2010-08-29 14:32:45 +0000
@@ -19,6 +19,7 @@
 from bzrlib import (
     branch as _mod_branch,
     bzrdir,
+    config,
     delta as _mod_delta,
     errors,
     gpg,
@@ -617,13 +618,9 @@
         self.assertEqual(None, self.get_branch().get_push_location())
 
     def test_get_push_location_exact(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        open(fn, 'wt').write(("[%s]\n"
-                                  "push_location=foo\n" %
-                                  self.get_branch().base[:-1]))
+        b = self.get_branch()
+        config.LocationConfig.from_string(
+            '[%s]\npush_location=foo\n' % (b.base,), b.base, save=True)
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2010-07-04 22:39:45 +0000
+++ bzrlib/tests/per_branch/test_push.py	2010-07-05 08:10:13 +0000
@@ -329,8 +329,9 @@
 
     def setUp(self):
         # Skip some scenarios that don't apply to these tests.
-        if (self.transport_server is not None and
-            issubclass(self.transport_server, server.SmartTCPServer)):
+        if (self.transport_server is not None
+            and issubclass(self.transport_server,
+                           test_server.SmartTCPServer_for_testing)):
             raise tests.TestNotApplicable(
                 'Does not apply when remote backing branch is also '
                 'a smart branch')

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2010-07-19 11:06:36 +0000
+++ bzrlib/tests/per_transport.py	2010-08-24 13:03:18 +0000
@@ -251,7 +251,6 @@
 
     def test_get_bytes_unknown_file(self):
         t = self.get_transport()
-
         self.assertRaises(NoSuchFile, t.get_bytes, 'c')
 
     def test_get_with_open_write_stream_sees_all_content(self):
@@ -1266,7 +1265,7 @@
         self.assertIs(t._get_connection(), c._get_connection())
 
         # Temporary failure, we need to create a new dummy connection
-        new_connection = object()
+        new_connection = None
         t._set_connection(new_connection)
         # Check that both transports use the same connection
         self.assertIs(new_connection, t._get_connection())

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-06-02 04:50:35 +0000
+++ bzrlib/tests/stub_sftp.py	2010-07-01 15:14:35 +0000
@@ -23,6 +23,7 @@
 import paramiko
 import select
 import socket
+import SocketServer
 import sys
 import threading
 import time
@@ -38,24 +39,24 @@
 from bzrlib.tests import test_server
 
 
-class StubServer (paramiko.ServerInterface):
+class StubServer(paramiko.ServerInterface):
 
-    def __init__(self, test_case):
+    def __init__(self, test_case_server):
         paramiko.ServerInterface.__init__(self)
-        self._test_case = test_case
+        self.log = test_case_server.log
 
     def check_auth_password(self, username, password):
         # all are allowed
-        self._test_case.log('sftpserver - authorizing: %s' % (username,))
+        self.log('sftpserver - authorizing: %s' % (username,))
         return paramiko.AUTH_SUCCESSFUL
 
     def check_channel_request(self, kind, chanid):
-        self._test_case.log(
-            'sftpserver - channel request: %s, %s' % (kind, chanid))
+        self.log('sftpserver - channel request: %s, %s' % (kind, chanid))
         return paramiko.OPEN_SUCCEEDED
 
 
-class StubSFTPHandle (paramiko.SFTPHandle):
+class StubSFTPHandle(paramiko.SFTPHandle):
+
     def stat(self):
         try:
             return paramiko.SFTPAttributes.from_stat(
@@ -73,7 +74,7 @@
             return paramiko.SFTPServer.convert_errno(e.errno)
 
 
-class StubSFTPServer (paramiko.SFTPServerInterface):
+class StubSFTPServer(paramiko.SFTPServerInterface):
 
     def __init__(self, server, root, home=None):
         paramiko.SFTPServerInterface.__init__(self, server)
@@ -90,7 +91,7 @@
             self.home = home[len(self.root):]
         if self.home.startswith('/'):
             self.home = self.home[1:]
-        server._test_case.log('sftpserver - new connection')
+        server.log('sftpserver - new connection')
 
     def _realpath(self, path):
         # paths returned from self.canonicalize() always start with
@@ -241,6 +242,7 @@
     # removed: chattr, symlink, readlink
     # (nothing in bzr's sftp transport uses those)
 
+
 # ------------- server test implementation --------------
 
 STUB_SERVER_KEY = """
@@ -262,55 +264,6 @@
 """
 
 
-class SocketListener(threading.Thread):
-
-    def __init__(self, callback):
-        threading.Thread.__init__(self)
-        self._callback = callback
-        self._socket = socket.socket()
-        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self._socket.bind(('localhost', 0))
-        self._socket.listen(1)
-        self.host, self.port = self._socket.getsockname()[:2]
-        self._stop_event = threading.Event()
-
-    def stop(self):
-        # called from outside this thread
-        self._stop_event.set()
-        # use a timeout here, because if the test fails, the server thread may
-        # never notice the stop_event.
-        self.join(5.0)
-        self._socket.close()
-
-    def run(self):
-        trace.mutter('SocketListener %r has started', self)
-        while True:
-            readable, writable_unused, exception_unused = \
-                select.select([self._socket], [], [], 0.1)
-            if self._stop_event.isSet():
-                trace.mutter('SocketListener %r has stopped', self)
-                return
-            if len(readable) == 0:
-                continue
-            try:
-                s, addr_unused = self._socket.accept()
-                trace.mutter('SocketListener %r has accepted connection %r',
-                    self, s)
-                # because the loopback socket is inline, and transports are
-                # never explicitly closed, best to launch a new thread.
-                threading.Thread(target=self._callback, args=(s,)).start()
-            except socket.error, x:
-                sys.excepthook(*sys.exc_info())
-                trace.warning('Socket error during accept() '
-                              'within unit test server thread: %r' % x)
-            except Exception, x:
-                # probably a failed test; unit test thread will log the
-                # failure/error
-                sys.excepthook(*sys.exc_info())
-                trace.warning(
-                    'Exception from within unit test server thread: %r' % x)
-
-
 class SocketDelay(object):
     """A socket decorator to make TCP appear slower.
 
@@ -386,55 +339,133 @@
         return bytes_sent
 
 
-class SFTPServer(test_server.TestServer):
+class TestingSFTPConnectionHandler(SocketServer.BaseRequestHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        tcs = self.server.test_case_server
+        ssh_server = paramiko.Transport(self.request)
+        ssh_server.add_server_key(tcs.get_host_key())
+        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
+                                         StubSFTPServer, root=tcs._root,
+                                         home=tcs._server_homedir)
+        server = tcs._server_interface(tcs)
+        ssh_server.start_server(None, server)
+        # FIXME: Long story short:
+        # bt.test_transport.TestSSHConnections.test_bzr_connect_to_bzr_ssh
+        # fails if we wait less than 0.2 seconds... paramiko uses a lot of
+        # timeouts internally which probably mask a synchronisation
+        # problem. Note that this is the only test that requires this hack and
+        # the test may need to be fixed instead, but it's late and the test is
+        # horrible as mentioned in its comments :) -- vila 20100623
+        import time
+        time.sleep(0.2)
+
+    def wrap_for_latency(self):
+        tcs = self.server.test_case_server
+        if tcs.add_latency:
+            # Give the socket (which the request really is) a latency adding
+            # decorator.
+            self.request = SocketDelay(self.request, tcs.add_latency)
+
+
+class TestingSFTPWithoutSSHConnectionHandler(TestingSFTPConnectionHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        # Re-import these as locals, so that they're still accessible during
+        # interpreter shutdown (when all module globals get set to None, leading
+        # to confusing errors like "'NoneType' object has no attribute 'error'".
+        class FakeChannel(object):
+            def get_transport(self):
+                return self
+            def get_log_channel(self):
+                return 'paramiko'
+            def get_name(self):
+                return '1'
+            def get_hexdump(self):
+                return False
+            def close(self):
+                pass
+
+        tcs = self.server.test_case_server
+        server = paramiko.SFTPServer(
+            FakeChannel(), 'sftp', StubServer(tcs), StubSFTPServer,
+            root=tcs._root, home=tcs._server_homedir)
+        try:
+            server.start_subsystem(
+                'sftp', None, ssh.SocketAsChannelAdapter(self.request))
+        except socket.error, e:
+            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
+                # it's okay for the client to disconnect abruptly
+                # (bug in paramiko 1.6: it should absorb this exception)
+                pass
+            else:
+                raise
+        except Exception, e:
+            # This typically seems to happen during interpreter shutdown, so
+            # most of the useful ways to report this error won't work.
+            # Writing the exception type, and then the text of the exception,
+            # seems to be the best we can do.
+            # FIXME: All interpreter shutdown errors should have been related
+            # to daemon threads, cleanup needed -- vila 20100623
+            import sys
+            sys.stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
+            sys.stderr.write('%s\n\n' % (e,))
+        server.finish_subsystem()
+
+
+class TestingSFTPServer(test_server.TestingThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class, test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(
+            self, server_address, request_handler_class)
+        self.test_case_server = test_case_server
+
+
+class SFTPServer(test_server.TestingTCPServerInAThread):
     """Common code for SFTP server facilities."""
 
     def __init__(self, server_interface=StubServer):
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SFTPServer, self).__init__((self.host, self.port),
+                                         TestingSFTPServer,
+                                         TestingSFTPConnectionHandler)
         self._original_vendor = None
-        self._homedir = None
-        self._server_homedir = None
-        self._listener = None
-        self._root = None
         self._vendor = ssh.ParamikoVendor()
         self._server_interface = server_interface
-        # sftp server logs
+        self._host_key = None
         self.logs = []
         self.add_latency = 0
+        self._homedir = None
+        self._server_homedir = None
+        self._root = None
 
     def _get_sftp_url(self, path):
         """Calculate an sftp url to this server for path."""
-        return 'sftp://foo:bar@%s:%d/%s' % (self._listener.host,
-                                            self._listener.port, path)
+        return "sftp://foo:bar@%s:%s/%s" % (self.host, self.port, path)
 
     def log(self, message):
         """StubServer uses this to log when a new server is created."""
         self.logs.append(message)
 
-    def _run_server_entry(self, sock):
-        """Entry point for all implementations of _run_server.
-
-        If self.add_latency is > 0.000001 then sock is given a latency adding
-        decorator.
-        """
-        if self.add_latency > 0.000001:
-            sock = SocketDelay(sock, self.add_latency)
-        return self._run_server(sock)
-
-    def _run_server(self, s):
-        ssh_server = paramiko.Transport(s)
-        key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
-        f = open(key_file, 'w')
-        f.write(STUB_SERVER_KEY)
-        f.close()
-        host_key = paramiko.RSAKey.from_private_key_file(key_file)
-        ssh_server.add_server_key(host_key)
-        server = self._server_interface(self)
-        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
-                                         StubSFTPServer, root=self._root,
-                                         home=self._server_homedir)
-        event = threading.Event()
-        ssh_server.start_server(event, server)
-        event.wait(5.0)
+    def create_server(self):
+        server = self.server_class((self.host, self.port),
+                                   self.request_handler_class,
+                                   self)
+        return server
+
+    def get_host_key(self):
+        if self._host_key is None:
+            key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
+            f = open(key_file, 'w')
+            try:
+                f.write(STUB_SERVER_KEY)
+            finally:
+                f.close()
+            self._host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        return self._host_key
 
     def start_server(self, backing_server=None):
         # XXX: TODO: make sftpserver back onto backing_server rather than local
@@ -459,17 +490,17 @@
         self._root = '/'
         if sys.platform == 'win32':
             self._root = ''
-        self._listener = SocketListener(self._run_server_entry)
-        self._listener.setDaemon(True)
-        self._listener.start()
+        super(SFTPServer, self).start_server()
 
     def stop_server(self):
-        self._listener.stop()
-        ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
+        try:
+            super(SFTPServer, self).stop_server()
+        finally:
+            ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
 
     def get_bogus_url(self):
         """See bzrlib.transport.Server.get_bogus_url."""
-        # this is chosen to try to prevent trouble with proxies, wierd dns, etc
+        # this is chosen to try to prevent trouble with proxies, weird dns, etc
         # we bind a random socket, so that we get a guaranteed unused port
         # we just never listen on that port
         s = socket.socket()
@@ -495,45 +526,10 @@
     def __init__(self):
         super(SFTPServerWithoutSSH, self).__init__()
         self._vendor = ssh.LoopbackVendor()
-
-    def _run_server(self, sock):
-        # Re-import these as locals, so that they're still accessible during
-        # interpreter shutdown (when all module globals get set to None, leading
-        # to confusing errors like "'NoneType' object has no attribute 'error'".
-        class FakeChannel(object):
-            def get_transport(self):
-                return self
-            def get_log_channel(self):
-                return 'paramiko'
-            def get_name(self):
-                return '1'
-            def get_hexdump(self):
-                return False
-            def close(self):
-                pass
-
-        server = paramiko.SFTPServer(
-            FakeChannel(), 'sftp', StubServer(self), StubSFTPServer,
-            root=self._root, home=self._server_homedir)
-        try:
-            server.start_subsystem(
-                'sftp', None, ssh.SocketAsChannelAdapter(sock))
-        except socket.error, e:
-            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
-                # it's okay for the client to disconnect abruptly
-                # (bug in paramiko 1.6: it should absorb this exception)
-                pass
-            else:
-                raise
-        except Exception, e:
-            # This typically seems to happen during interpreter shutdown, so
-            # most of the useful ways to report this error are won't work.
-            # Writing the exception type, and then the text of the exception,
-            # seems to be the best we can do.
-            import sys
-            sys.stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
-            sys.stderr.write('%s\n\n' % (e,))
-        server.finish_subsystem()
+        self.request_handler_class = TestingSFTPWithoutSSHConnectionHandler
+
+    def get_host_key():
+        return None
 
 
 class SFTPAbsoluteServer(SFTPServerWithoutSSH):
@@ -562,7 +558,9 @@
     It does this by serving from a deeply-nested directory that doesn't exist.
     """
 
-    def start_server(self, backing_server=None):
-        self._server_homedir = '/dev/noone/runs/tests/here'
-        super(SFTPSiblingAbsoluteServer, self).start_server(backing_server)
+    def create_server(self):
+        # FIXME: Can't we do that in a cleaner way ? -- vila 20100623
+        server = super(SFTPSiblingAbsoluteServer, self).create_server()
+        server._server_homedir = '/dev/noone/runs/tests/here'
+        return server
 

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2010-08-13 07:32:06 +0000
+++ bzrlib/tests/test_branch.py	2010-08-29 14:32:45 +0000
@@ -86,18 +86,7 @@
         self.assertIsDirectory('.bzr/branch/lock/held', t)
 
     def test_set_push_location(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        # write correct newlines to locations.conf
-        # by default ConfigObj uses native line-endings for new files
-        # but uses already existing line-endings if file is not empty
-        f = open(fn, 'wb')
-        try:
-            f.write('# comment\n')
-        finally:
-            f.close()
+        conf = config.LocationConfig.from_string('# comment\n', '.', save=True)
 
         branch = self.make_branch('.', format='knit')
         branch.set_push_location('foo')
@@ -106,7 +95,7 @@
                              "[%s]\n"
                              "push_location = foo\n"
                              "push_location:policy = norecurse\n" % local_path,
-                             fn)
+                             config.locations_config_filename())
 
     # TODO RBC 20051029 test getting a push location from a branch in a
     # recursive section - that is, it appends the branch name.

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2010-08-13 08:09:53 +0000
+++ bzrlib/tests/test_commands.py	2010-08-28 16:56:36 +0000
@@ -95,9 +95,7 @@
 class TestGetAlias(tests.TestCase):
 
     def _get_config(self, config_text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(config_text.encode('utf-8'))
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(config_text)
         return my_config
 
     def test_simple(self):

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2010-07-21 03:07:02 +0000
+++ bzrlib/tests/test_config.py	2010-08-30 08:26:45 +0000
@@ -19,6 +19,7 @@
 from cStringIO import StringIO
 import os
 import sys
+import threading
 
 #import bzrlib specific imports here
 from bzrlib import (
@@ -39,6 +40,30 @@
 from bzrlib.util.configobj import configobj
 
 
+def lockable_config_scenarios():
+    return [
+        ('global',
+         {'config_class': config.GlobalConfig,
+          'config_args': [],
+          'config_section': 'DEFAULT'}),
+        ('locations',
+         {'config_class': config.LocationConfig,
+          'config_args': ['.'],
+          'config_section': '.'}),]
+
+
+def load_tests(standard_tests, module, loader):
+    suite = loader.suiteClass()
+
+    lc_tests, remaining_tests = tests.split_suite_by_condition(
+        standard_tests, tests.condition_isinstance((
+                TestLockableConfig,
+                )))
+    tests.multiply_tests(lc_tests, lockable_config_scenarios(), suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
 sample_long_alias="log -r-15..-1 --line"
 sample_config_text = u"""
 [DEFAULT]
@@ -130,6 +155,9 @@
         self._calls.append(('keys',))
         return []
 
+    def reload(self):
+        self._calls.append(('reload',))
+
     def write(self, arg):
         self._calls.append(('write',))
 
@@ -351,10 +379,6 @@
         self.assertEqual(config.config_filename(),
                          self.bzr_home + '/bazaar.conf')
 
-    def test_branches_config_filename(self):
-        self.assertEqual(config.branches_config_filename(),
-                         self.bzr_home + '/branches.conf')
-
     def test_locations_config_filename(self):
         self.assertEqual(config.locations_config_filename(),
                          self.bzr_home + '/locations.conf')
@@ -371,46 +395,211 @@
 class TestIniConfig(tests.TestCaseInTempDir):
 
     def make_config_parser(self, s):
-        conf = config.IniBasedConfig(None)
-        parser = conf._get_parser(file=StringIO(s.encode('utf-8')))
-        return conf, parser
+        conf = config.IniBasedConfig.from_string(s)
+        return conf, conf._get_parser()
 
 
 class TestIniConfigBuilding(TestIniConfig):
 
     def test_contructs(self):
-        my_config = config.IniBasedConfig("nothing")
+        my_config = config.IniBasedConfig()
 
     def test_from_fp(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        self.failUnless(
-            isinstance(my_config._get_parser(file=config_file),
-                        configobj.ConfigObj))
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertIsInstance(my_config._get_parser(), configobj.ConfigObj)
 
     def test_cached(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        parser = my_config._get_parser(file=config_file)
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        parser = my_config._get_parser()
         self.failUnless(my_config._get_parser() is parser)
 
     def _dummy_chown(self, path, uid, gid):
         self.path, self.uid, self.gid = path, uid, gid
 
     def test_ini_config_ownership(self):
-        """Ensure that chown is happening during _write_config_file.
-        """
+        """Ensure that chown is happening during _write_config_file"""
         self.requireFeature(features.chown_feature)
         self.overrideAttr(os, 'chown', self._dummy_chown)
         self.path = self.uid = self.gid = None
-        def get_filename():
-            return 'foo.conf'
-        conf = config.IniBasedConfig(get_filename)
+        conf = config.IniBasedConfig(file_name='./foo.conf')
         conf._write_config_file()
-        self.assertEquals(self.path, 'foo.conf')
+        self.assertEquals(self.path, './foo.conf')
         self.assertTrue(isinstance(self.uid, int))
         self.assertTrue(isinstance(self.gid, int))
 
+    def test_get_filename_parameter_is_deprecated_(self):
+        conf = self.callDeprecated([
+            'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+            ' Use file_name instead.'],
+            config.IniBasedConfig, lambda: 'ini.conf')
+        self.assertEqual('ini.conf', conf.file_name)
+
+    def test_get_parser_file_parameter_is_deprecated_(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        conf = self.callDeprecated([
+            'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+            ' Use IniBasedConfig(_content=xxx) instead.'],
+            conf._get_parser, file=config_file)
+
+class TestIniConfigSaving(tests.TestCaseInTempDir):
+
+    def test_cant_save_without_a_file_name(self):
+        conf = config.IniBasedConfig()
+        self.assertRaises(AssertionError, conf._write_config_file)
+
+    def test_saved_with_content(self):
+        content = 'foo = bar\n'
+        conf = config.IniBasedConfig.from_string(
+            content, file_name='./test.conf', save=True)
+        self.assertFileEqual(content, 'test.conf')
+
+
+class TestIniBaseConfigOnDisk(tests.TestCaseInTempDir):
+
+    def test_cannot_reload_without_name(self):
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertRaises(AssertionError, conf.reload)
+
+    def test_reload_see_new_value(self):
+        c1 = config.IniBasedConfig.from_string('editor=vim\n',
+                                               file_name='./test/conf')
+        c1._write_config_file()
+        c2 = config.IniBasedConfig.from_string('editor=emacs\n',
+                                               file_name='./test/conf')
+        c2._write_config_file()
+        self.assertEqual('vim', c1.get_user_option('editor'))
+        self.assertEqual('emacs', c2.get_user_option('editor'))
+        # Make sure we get the Right value
+        c1.reload()
+        self.assertEqual('emacs', c1.get_user_option('editor'))
+
+
+class TestLockableConfig(tests.TestCaseInTempDir):
+
+    # Set by load_tests
+    config_class = None
+    config_args = None
+    config_section = None
+
+    def setUp(self):
+        super(TestLockableConfig, self).setUp()
+        self._content = '[%s]\none=1\ntwo=2\n' % (self.config_section,)
+        self.config = self.create_config(self._content)
+
+    def get_existing_config(self):
+        return self.config_class(*self.config_args)
+
+    def create_config(self, content):
+        kwargs = dict(save=True)
+        c = self.config_class.from_string(content, *self.config_args, **kwargs)
+        return c
+
+    def test_simple_read_access(self):
+        self.assertEquals('1', self.config.get_user_option('one'))
+
+    def test_simple_write_access(self):
+        self.config.set_user_option('one', 'one')
+        self.assertEquals('one', self.config.get_user_option('one'))
+
+    def test_listen_to_the_last_speaker(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'ONE')
+        c2.set_user_option('two', 'TWO')
+        self.assertEquals('ONE', c1.get_user_option('one'))
+        self.assertEquals('TWO', c2.get_user_option('two'))
+        # The second update respect the first one
+        self.assertEquals('ONE', c2.get_user_option('one'))
+
+    def test_last_speaker_wins(self):
+        # If the same config is not shared, the same variable modified twice
+        # can only see a single result.
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'c1')
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2._get_user_option('one'))
+        # The first modification is still available until another refresh
+        # occur
+        self.assertEquals('c1', c1._get_user_option('one'))
+        c1.set_user_option('two', 'done')
+        self.assertEquals('c2', c1._get_user_option('one'))
+
+    def test_writes_are_serialized(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+
+        # We spawn a thread that will pause *during* the write
+        before_writing = threading.Event()
+        after_writing = threading.Event()
+        writing_done = threading.Event()
+        c1_orig = c1._write_config_file
+        def c1_write_config_file():
+            before_writing.set()
+            c1_orig()
+            # The lock is held we wait for the main thread to decide when to
+            # continue
+            after_writing.wait()
+        c1._write_config_file = c1_write_config_file
+        def c1_set_option():
+            c1.set_user_option('one', 'c1')
+            writing_done.set()
+        t1 = threading.Thread(target=c1_set_option)
+        # Collect the thread after the test
+        self.addCleanup(t1.join)
+        # Be ready to unblock the thread if the test goes wrong
+        self.addCleanup(after_writing.set)
+        t1.start()
+        before_writing.wait()
+        self.assertTrue(c1._lock.is_held)
+        self.assertRaises(errors.LockContention,
+                          c2.set_user_option, 'one', 'c2')
+        self.assertEquals('c1', c1.get_user_option('one'))
+        # Let the lock be released
+        after_writing.set()
+        writing_done.wait()
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2.get_user_option('one'))
+
+    def test_read_while_writing(self):
+       c1 = self.config
+       # We spawn a thread that will pause *during* the write
+       ready_to_write = threading.Event()
+       do_writing = threading.Event()
+       writing_done = threading.Event()
+       c1_orig = c1._write_config_file
+       def c1_write_config_file():
+           ready_to_write.set()
+           # The lock is held we wait for the main thread to decide when to
+           # continue
+           do_writing.wait()
+           c1_orig()
+           writing_done.set()
+       c1._write_config_file = c1_write_config_file
+       def c1_set_option():
+           c1.set_user_option('one', 'c1')
+       t1 = threading.Thread(target=c1_set_option)
+       # Collect the thread after the test
+       self.addCleanup(t1.join)
+       # Be ready to unblock the thread if the test goes wrong
+       self.addCleanup(do_writing.set)
+       t1.start()
+       # Ensure the thread is ready to write
+       ready_to_write.wait()
+       self.assertTrue(c1._lock.is_held)
+       self.assertEquals('c1', c1.get_user_option('one'))
+       # If we read during the write, we get the old value
+       c2 = self.get_existing_config()
+       self.assertEquals('1', c2.get_user_option('one'))
+       # Let the writing occur and ensure it occurred
+       do_writing.set()
+       writing_done.wait()
+       # Now we get the updated value
+       c3 = self.get_existing_config()
+       self.assertEquals('c1', c3.get_user_option('one'))
+
+
 class TestGetUserOptionAs(TestIniConfig):
 
     def test_get_user_option_as_bool(self):
@@ -522,11 +711,10 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
         local_url = urlutils.local_path_to_url('branch')
-        open(locations, 'wb').write('[%s]\nnickname = foobar'
-                                    % (local_url,))
+        conf = config.LocationConfig.from_string(
+            '[%s]\nnickname = foobar' % (local_url,),
+            local_url, save=True)
         self.assertEqual('foobar', branch.nick)
 
     def test_config_local_path(self):
@@ -534,20 +722,19 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
-        open(locations, 'wb').write('[%s/branch]\nnickname = barry'
-                                    % (osutils.getcwd().encode('utf8'),))
+        local_path = osutils.getcwd().encode('utf8')
+        conf = config.LocationConfig.from_string(
+            '[%s/branch]\nnickname = barry' % (local_path,),
+            'branch',  save=True)
         self.assertEqual('barry', branch.nick)
 
     def test_config_creates_local(self):
         """Creating a new entry in config uses a local path."""
         branch = self.make_branch('branch', format='knit')
         branch.set_push_location('http://foobar')
-        locations = config.locations_config_filename()
         local_path = osutils.getcwd().encode('utf8')
         # Surprisingly ConfigObj doesn't create a trailing newline
-        self.check_file_contents(locations,
+        self.check_file_contents(config.locations_config_filename(),
                                  '[%s/branch]\n'
                                  'push_location = http://foobar\n'
                                  'push_location:policy = norecurse\n'
@@ -558,10 +745,10 @@
         self.assertEqual('!repo', b.get_config().get_nickname())
 
     def test_warn_if_masked(self):
-        _warning = trace.warning
         warnings = []
         def warning(*args):
             warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
 
         def set_option(store, warn_masked=True):
             warnings[:] = []
@@ -573,53 +760,41 @@
             else:
                 self.assertEqual(1, len(warnings))
                 self.assertEqual(warning, warnings[0])
-        trace.warning = warning
-        try:
-            branch = self.make_branch('.')
-            conf = branch.get_config()
-            set_option(config.STORE_GLOBAL)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning(None)
-            set_option(config.STORE_GLOBAL)
-            assertWarning('Value "4" is masked by "3" from branch.conf')
-            set_option(config.STORE_GLOBAL, warn_masked=False)
-            assertWarning(None)
-            set_option(config.STORE_LOCATION)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning('Value "3" is masked by "0" from locations.conf')
-            set_option(config.STORE_BRANCH, warn_masked=False)
-            assertWarning(None)
-        finally:
-            trace.warning = _warning
+        branch = self.make_branch('.')
+        conf = branch.get_config()
+        set_option(config.STORE_GLOBAL)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning(None)
+        set_option(config.STORE_GLOBAL)
+        assertWarning('Value "4" is masked by "3" from branch.conf')
+        set_option(config.STORE_GLOBAL, warn_masked=False)
+        assertWarning(None)
+        set_option(config.STORE_LOCATION)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning('Value "3" is masked by "0" from locations.conf')
+        set_option(config.STORE_BRANCH, warn_masked=False)
+        assertWarning(None)
 
 
 class TestGlobalConfigItems(tests.TestCase):
 
     def test_user_id(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
 
     def test_absent_user_id(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         self.assertEqual(None, my_config._get_user_id())
 
     def test_configured_editor(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         self.assertEqual("vim", my_config.get_editor())
 
     def test_signatures_always(self):
-        config_file = StringIO(sample_always_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_always_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_ALWAYS,
@@ -627,9 +802,7 @@
         self.assertEqual(True, my_config.signature_needed())
 
     def test_signatures_if_possible(self):
-        config_file = StringIO(sample_maybe_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_maybe_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_WHEN_REQUIRED,
@@ -637,9 +810,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def test_signatures_ignore(self):
-        config_file = StringIO(sample_ignore_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_ignore_signatures)
         self.assertEqual(config.CHECK_ALWAYS,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_NEVER,
@@ -647,9 +818,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_sample_config(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         return my_config
 
     def test_gpg_signing_command(self):
@@ -658,9 +827,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_empty_config(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         return my_config
 
     def test_gpg_signing_command_unset(self):
@@ -761,18 +928,6 @@
         self.assertEqual(parser._calls,
                          [('__init__', config.locations_config_filename(),
                            'utf-8')])
-        config.ensure_config_dir_exists()
-        #os.mkdir(config.config_dir())
-        f = file(config.branches_config_filename(), 'wb')
-        f.write('')
-        f.close()
-        oldparserclass = config.ConfigObj
-        config.ConfigObj = InstrumentedConfigObj
-        try:
-            my_config = config.LocationConfig('http://www.example.com')
-            parser = my_config._get_parser()
-        finally:
-            config.ConfigObj = oldparserclass
 
     def test_get_global_config(self):
         my_config = config.BranchConfig(FakeBranch('http://example.com'))
@@ -1007,42 +1162,30 @@
                          self.my_config.post_commit())
 
     def get_branch_config(self, location, global_config=None):
+        my_branch = FakeBranch(location)
         if global_config is None:
-            global_file = StringIO(sample_config_text.encode('utf-8'))
-        else:
-            global_file = StringIO(global_config.encode('utf-8'))
-        branches_file = StringIO(sample_branches_text.encode('utf-8'))
-        self.my_config = config.BranchConfig(FakeBranch(location))
-        # Force location config to use specified file
-        self.my_location_config = self.my_config._get_location_config()
-        self.my_location_config._get_parser(branches_file)
-        # Force global config to use specified file
-        self.my_config._get_global_config()._get_parser(global_file)
+            global_config = sample_config_text
+
+        my_global_config = config.GlobalConfig.from_string(global_config,
+                                                           save=True)
+        my_location_config = config.LocationConfig.from_string(
+            sample_branches_text, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
+        self.my_config = my_config
+        self.my_location_config = my_config._get_location_config()
 
     def test_set_user_setting_sets_and_saves(self):
         self.get_branch_config('/a/c')
         record = InstrumentedConfigObj("foo")
         self.my_location_config._parser = record
 
-        real_mkdir = os.mkdir
-        self.created = False
-        def checked_mkdir(path, mode=0777):
-            self.log('making directory: %s', path)
-            real_mkdir(path, mode)
-            self.created = True
-
-        os.mkdir = checked_mkdir
-        try:
-            self.callDeprecated(['The recurse option is deprecated as of '
-                                 '0.14.  The section "/a/c" has been '
-                                 'converted to use policies.'],
-                                self.my_config.set_user_option,
-                                'foo', 'bar', store=config.STORE_LOCATION)
-        finally:
-            os.mkdir = real_mkdir
-
-        self.failUnless(self.created, 'Failed to create ~/.bazaar')
-        self.assertEqual([('__contains__', '/a/c'),
+        self.callDeprecated(['The recurse option is deprecated as of '
+                             '0.14.  The section "/a/c" has been '
+                             'converted to use policies.'],
+                            self.my_config.set_user_option,
+                            'foo', 'bar', store=config.STORE_LOCATION)
+        self.assertEqual([('reload',),
+                          ('__contains__', '/a/c'),
                           ('__contains__', '/a/c/'),
                           ('__setitem__', '/a/c', {}),
                           ('__getitem__', '/a/c'),
@@ -1091,19 +1234,18 @@
 option = exact
 """
 
-
 class TestBranchConfigItems(tests.TestCaseInTempDir):
 
     def get_branch_config(self, global_config=None, location=None,
                           location_config=None, branch_data_config=None):
-        my_config = config.BranchConfig(FakeBranch(location))
+        my_branch = FakeBranch(location)
         if global_config is not None:
-            global_file = StringIO(global_config.encode('utf-8'))
-            my_config._get_global_config()._get_parser(global_file)
-        self.my_location_config = my_config._get_location_config()
+            my_global_config = config.GlobalConfig.from_string(global_config,
+                                                               save=True)
         if location_config is not None:
-            location_file = StringIO(location_config.encode('utf-8'))
-            self.my_location_config._get_parser(location_file)
+            my_location_config = config.LocationConfig.from_string(
+                location_config, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
         if branch_data_config is not None:
             my_config.branch.control_files.files['branch.conf'] = \
                 branch_data_config
@@ -1123,7 +1265,7 @@
                          my_config.username())
 
     def test_not_set_in_branch(self):
-        my_config = self.get_branch_config(sample_config_text)
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
         my_config.branch.control_files.files['email'] = "John"
@@ -1153,29 +1295,25 @@
 
     def test_gpg_signing_command(self):
         my_config = self.get_branch_config(
+            global_config=sample_config_text,
             # branch data cannot set gpg_signing_command
             branch_data_config="gpg_signing_command=pgp")
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config._get_global_config()._get_parser(config_file)
         self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
 
     def test_get_user_option_global(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_global_config()._get_parser(config_file))
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual('something',
                          my_config.get_user_option('user_global_option'))
 
     def test_post_commit_default(self):
-        branch = FakeBranch()
-        my_config = self.get_branch_config(sample_config_text, '/a/c',
-                                           sample_branches_text)
+        my_config = self.get_branch_config(global_config=sample_config_text,
+                                      location='/a/c',
+                                      location_config=sample_branches_text)
         self.assertEqual(my_config.branch.base, '/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root')
-        # post-commit is ignored when bresent in branch data
+        # post-commit is ignored when present in branch data
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root',
@@ -1183,19 +1321,23 @@
         self.assertEqual('rmtree_root', my_config.post_commit())
 
     def test_config_precedence(self):
+        # FIXME: eager test, luckily no persitent config file makes it fail
+        # -- vila 20100716
         my_config = self.get_branch_config(global_config=precedence_global)
         self.assertEqual(my_config.get_user_option('option'), 'global')
         my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch)
+                                           branch_data_config=precedence_branch)
         self.assertEqual(my_config.get_user_option('option'), 'branch')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location)
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location)
         self.assertEqual(my_config.get_user_option('option'), 'recurse')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location,
-                                      location='http://example.com/specific')
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location,
+            location='http://example.com/specific')
         self.assertEqual(my_config.get_user_option('option'), 'exact')
 
     def test_get_mail_client(self):

=== modified file 'bzrlib/tests/test_debug.py'
--- bzrlib/tests/test_debug.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_debug.py	2010-08-29 14:32:45 +0000
@@ -17,14 +17,11 @@
 """Tests for bzrlib.debug"""
 
 
-import os
-
-
 from bzrlib import (
+    config,
     debug,
     tests,
     )
-from bzrlib.config import config_filename, ensure_config_dir_exists
 
 
 class TestDebugFlags(tests.TestCaseInTempDir):
@@ -32,33 +29,11 @@
     def test_set_debug_flags_from_config(self):
         # test both combinations because configobject automatically splits up
         # comma-separated lists
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        self.try_debug_flags(
-            """debug_flags = hpss, error\n""",
-            set(['hpss', 'error']))
-
-        self.try_debug_flags(
-            """debug_flags = hpss\n""",
-            set(['hpss']))
-
-    def try_debug_flags(self, conf_bytes, expected_flags):
-        ensure_config_dir_exists()
-        f = open(config_filename(), 'wb')
-        try:
-            f.write(conf_bytes)
-        finally:
-            f.close()
-        saved_debug = set(debug.debug_flags)
-        debug.debug_flags.clear()
-        try:
-            debug.set_debug_flags_from_config()
-            self.assertEqual(expected_flags,
-                debug.debug_flags)
-        finally:
-            # restore without rebinding the variable
-            debug.debug_flags.clear()
-            debug.debug_flags.update(saved_debug)
+        self.try_debug_flags(['hpss', 'error'], 'debug_flags = hpss, error\n')
+        self.try_debug_flags(['hpss'], 'debug_flags = hpss\n')
+
+    def try_debug_flags(self, expected_flags, conf_bytes):
+        conf = config.GlobalConfig.from_string(conf_bytes, save=True)
+        self.overrideAttr(debug, 'debug_flags', set())
+        debug.set_debug_flags_from_config()
+        self.assertEqual(set(expected_flags), debug.debug_flags)

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2010-07-15 12:53:44 +0000
+++ bzrlib/tests/test_http.py	2010-08-24 13:03:18 +0000
@@ -48,6 +48,7 @@
     features,
     http_server,
     http_utils,
+    test_server,
     )
 from bzrlib.transport import (
     http,
@@ -77,13 +78,13 @@
     transport_scenarios = [
         ('urllib', dict(_transport=_urllib.HttpTransport_urllib,
                         _server=http_server.HttpServer_urllib,
-                        _qualified_prefix='http+urllib',)),
+                        _url_protocol='http+urllib',)),
         ]
     if features.pycurl.available():
         transport_scenarios.append(
             ('pycurl', dict(_transport=PyCurlTransport,
                             _server=http_server.HttpServer_PyCurl,
-                            _qualified_prefix='http+pycurl',)))
+                            _url_protocol='http+pycurl',)))
     tests.multiply_tests(t_tests, transport_scenarios, result)
 
     protocol_scenarios = [
@@ -228,25 +229,22 @@
         self._sock.bind(('127.0.0.1', 0))
         self.host, self.port = self._sock.getsockname()
         self._ready = threading.Event()
-        self._thread = threading.Thread(target=self._accept_read_and_reply)
-        self._thread.setDaemon(True)
+        self._thread = test_server.ThreadWithException(
+            event=self._ready, target=self._accept_read_and_reply)
         self._thread.start()
-        self._ready.wait(5)
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n' % (self._thread.ident,))
+        self._ready.wait()
 
     def _accept_read_and_reply(self):
         self._sock.listen(1)
         self._ready.set()
-        self._sock.settimeout(5)
-        try:
-            conn, address = self._sock.accept()
-            # On win32, the accepted connection will be non-blocking to start
-            # with because we're using settimeout.
-            conn.setblocking(True)
+        conn, address = self._sock.accept()
+        if self._expect_body_tail is not None:
             while not self.received_bytes.endswith(self._expect_body_tail):
                 self.received_bytes += conn.recv(4096)
             conn.sendall('HTTP/1.1 200 OK\r\n')
-        except socket.timeout:
-            # Make sure the client isn't stuck waiting for us to e.g. accept.
+        try:
             self._sock.close()
         except socket.error:
             # The client may have already closed the socket.
@@ -254,12 +252,18 @@
 
     def stop_server(self):
         try:
-            self._sock.close()
+            # Issue a fake connection to wake up the server and allow it to
+            # finish quickly
+            fake_conn = osutils.connect_socket((self.host, self.port))
+            fake_conn.close()
         except socket.error:
             # We might have already closed it.  We don't care.
             pass
         self.host = None
         self.port = None
+        self._thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n' % (self._thread.ident,))
 
 
 class TestAuthHeader(tests.TestCase):
@@ -309,29 +313,20 @@
 
             protocol_version = 'HTTP/0.1'
 
-        server = http_server.HttpServer(BogusRequestHandler)
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, BogusRequestHandler)
 
     def test_force_invalid_protocol(self):
-        server = http_server.HttpServer(protocol_version='HTTP/0.1')
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, protocol_version='HTTP/0.1')
 
     def test_server_start_and_stop(self):
         server = http_server.HttpServer()
+        self.addCleanup(server.stop_server)
         server.start_server()
-        try:
-            self.assertTrue(server._http_running)
-        finally:
-            server.stop_server()
-        self.assertFalse(server._http_running)
+        self.assertTrue(server.server is not None)
+        self.assertTrue(server.server.serving is not None)
+        self.assertTrue(server.server.serving)
 
     def test_create_http_server_one_zero(self):
         class RequestHandlerOneZero(http_server.TestingHTTPRequestHandler):
@@ -340,7 +335,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneZero)
         self.start_server(server)
-        self.assertIsInstance(server._httpd, http_server.TestingHTTPServer)
+        self.assertIsInstance(server.server, http_server.TestingHTTPServer)
 
     def test_create_http_server_one_one(self):
         class RequestHandlerOneOne(http_server.TestingHTTPRequestHandler):
@@ -349,7 +344,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneOne)
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_one(self):
@@ -360,7 +355,7 @@
         server = http_server.HttpServer(RequestHandlerOneZero,
                                         protocol_version='HTTP/1.1')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_zero(self):
@@ -371,7 +366,7 @@
         server = http_server.HttpServer(RequestHandlerOneOne,
                                         protocol_version='HTTP/1.0')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingHTTPServer)
 
 
@@ -436,7 +431,7 @@
         server.start_server()
         try:
             url = server.get_url()
-            self.assertTrue(url.startswith('%s://' % self._qualified_prefix))
+            self.assertTrue(url.startswith('%s://' % self._url_protocol))
         finally:
             server.stop_server()
 
@@ -487,7 +482,7 @@
 
     def test_http_has(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('foo/bar'), True)
         self.assertEqual(len(server.logs), 1)
         self.assertContainsRe(server.logs[0],
@@ -495,14 +490,14 @@
 
     def test_http_has_not_found(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('not-found'), False)
         self.assertContainsRe(server.logs[1],
             r'"HEAD /not-found HTTP/1.." 404 - "-" "bzr/')
 
     def test_http_get(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         fp = t.get('foo/bar')
         self.assertEqualDiff(
             fp.read(),
@@ -531,7 +526,7 @@
     """Test registrations of various http implementations"""
 
     def test_http_registered(self):
-        t = transport.get_transport('%s://foo.com/' % self._qualified_prefix)
+        t = transport.get_transport('%s://foo.com/' % self._url_protocol)
         self.assertIsInstance(t, transport.Transport)
         self.assertIsInstance(t, self._transport)
 
@@ -540,10 +535,11 @@
 
     def test_post_body_is_received(self):
         server = RecordingServer(expect_body_tail='end-of-body',
-            scheme=self._qualified_prefix)
+                                 scheme=self._url_protocol)
         self.start_server(server)
         url = server.get_url()
-        http_transport = self._transport(url)
+        # FIXME: needs a cleanup -- vila 20100611
+        http_transport = transport.get_transport(url)
         code, response = http_transport._post('abc def end-of-body')
         self.assertTrue(
             server.received_bytes.startswith('POST /.bzr/smart HTTP/1.'))
@@ -593,8 +589,10 @@
     _req_handler_class = http_server.TestingHTTPRequestHandler
 
     def create_transport_readonly_server(self):
-        return http_server.HttpServer(self._req_handler_class,
-                                      protocol_version=self._protocol_version)
+        server = http_server.HttpServer(self._req_handler_class,
+                                        protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -605,7 +603,7 @@
 class WallRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, close the connection"""
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         """Handle a single HTTP request, by abruptly closing the connection"""
         self.close_connection = 1
 
@@ -616,8 +614,7 @@
     _req_handler_class = WallRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Unfortunately httplib (see HTTPResponse._read_status
         # for details) make no distinction between a closed
         # socket and badly formatted status line, so we can't
@@ -629,8 +626,7 @@
                           t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises((errors.ConnectionError, errors.ConnectionReset,
                            errors.InvalidHttpResponse),
                           t.get, 'foo/bar')
@@ -653,13 +649,11 @@
     _req_handler_class = BadStatusRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -670,6 +664,10 @@
         """Fakes handling a single HTTP request, returns a bad status"""
         ignored = http_server.TestingHTTPRequestHandler.parse_request(self)
         self.wfile.write("Invalid status line\r\n")
+        # If we don't close the connection pycurl will hang. Since this is a
+        # stress test we don't *have* to respect the protocol, but we don't
+        # have to sabotage it too much either.
+        self.close_connection = True
         return False
 
 
@@ -681,18 +679,6 @@
 
     _req_handler_class = InvalidStatusRequestHandler
 
-    def test_http_has(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_has()
-
-    def test_http_get(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_get()
-
 
 class BadProtocolRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, returns a bad protocol version"""
@@ -720,13 +706,11 @@
         super(TestBadProtocolServer, self).setUp()
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -746,13 +730,11 @@
     _req_handler_class = ForbiddenRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.get, 'foo/bar')
 
 
@@ -797,8 +779,7 @@
         self.build_tree_contents([('a', '0123456789')],)
 
     def test_readv(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
         self.assertEqual(l[0], (0, '0'))
         self.assertEqual(l[1], (1, '1'))
@@ -806,8 +787,7 @@
         self.assertEqual(l[3], (9, '9'))
 
     def test_readv_out_of_order(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((1, 1), (9, 1), (0, 1), (3, 2))))
         self.assertEqual(l[0], (1, '1'))
         self.assertEqual(l[1], (9, '9'))
@@ -815,8 +795,7 @@
         self.assertEqual(l[3], (3, '34'))
 
     def test_readv_invalid_ranges(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
@@ -830,7 +809,7 @@
 
     def test_readv_multiple_get_requests(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._max_readv_combine = 1
         t._max_get_ranges = 1
@@ -844,7 +823,7 @@
 
     def test_readv_get_max_size(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests by limiting the number of
         # bytes by request. Note that this apply to coalesced offsets only, a
         # single range will keep its size even if bigger than the limit.
@@ -859,7 +838,7 @@
 
     def test_complete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
         l = list(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
@@ -870,7 +849,7 @@
 
     def test_incomplete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
         # Don't collapse readv results into a list so that we leave unread
@@ -1026,7 +1005,7 @@
 
     def test_readv_with_short_reads(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Force separate ranges for each offset
         t._bytes_to_read_before_seek = 0
         ireadv = iter(t.readv('a', ((0, 1), (2, 1), (4, 2), (9, 1))))
@@ -1083,9 +1062,6 @@
         return LimitedRangeHTTPServer(range_limit=self.range_limit,
                                       protocol_version=self._protocol_version)
 
-    def get_transport(self):
-        return self._transport(self.get_readonly_server().get_url())
-
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         # We need to manipulate ranges that correspond to real chunks in the
@@ -1095,14 +1071,14 @@
         self.build_tree_contents([('a', content)],)
 
     def test_few_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), )))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
         self.assertEqual(1, self.get_readonly_server().GET_request_nb)
 
     def test_more_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), (4096, 4), (8192, 4))))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
@@ -1159,24 +1135,24 @@
     """
 
     # FIXME: We don't have an https server available, so we don't
-    # test https connections.
+    # test https connections. --vila toolongago
 
     def setUp(self):
         super(TestProxyHttpServer, self).setUp()
+        self.transport_secondary_server = http_utils.ProxyServer
         self.build_tree_contents([('foo', 'contents of foo\n'),
                                   ('foo-proxied', 'proxied contents of foo\n')])
         # Let's setup some attributes for tests
-        self.server = self.get_readonly_server()
-        self.proxy_address = '%s:%d' % (self.server.host, self.server.port)
+        server = self.get_readonly_server()
+        self.server_host_port = '%s:%d' % (server.host, server.port)
         if self._testing_pycurl():
             # Oh my ! pycurl does not check for the port as part of
             # no_proxy :-( So we just test the host part
-            self.no_proxy_host = self.server.host
+            self.no_proxy_host = server.host
         else:
-            self.no_proxy_host = self.proxy_address
+            self.no_proxy_host = self.server_host_port
         # The secondary server is the proxy
-        self.proxy = self.get_secondary_server()
-        self.proxy_url = self.proxy.get_url()
+        self.proxy_url = self.get_secondary_url()
         self._old_env = {}
 
     def _testing_pycurl(self):
@@ -1184,12 +1160,6 @@
         return (features.pycurl.available()
                 and self._transport == PyCurlTransport)
 
-    def create_transport_secondary_server(self):
-        """Creates an http server that will serve files with
-        '-proxied' appended to their names.
-        """
-        return http_utils.ProxyServer(protocol_version=self._protocol_version)
-
     def _install_env(self, env):
         for name, value in env.iteritems():
             self._old_env[name] = osutils.set_or_unset_env(name, value)
@@ -1200,8 +1170,7 @@
 
     def proxied_in_env(self, env):
         self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
+        t = self.get_readonly_transport()
         try:
             self.assertEqual('proxied contents of foo\n', t.get('foo').read())
         finally:
@@ -1209,8 +1178,7 @@
 
     def not_proxied_in_env(self, env):
         self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
+        t = self.get_readonly_transport()
         try:
             self.assertEqual('contents of foo\n', t.get('foo').read())
         finally:
@@ -1258,11 +1226,11 @@
             # pycurl *ignores* invalid proxy env variables. If that ever change
             # in the future, this test will fail indicating that pycurl do not
             # ignore anymore such variables.
-            self.not_proxied_in_env({'http_proxy': self.proxy_address})
+            self.not_proxied_in_env({'http_proxy': self.server_host_port})
         else:
             self.assertRaises(errors.InvalidURL,
                               self.proxied_in_env,
-                              {'http_proxy': self.proxy_address})
+                              {'http_proxy': self.server_host_port})
 
 
 class TestRanges(http_utils.TestCaseWithWebserver):
@@ -1271,24 +1239,24 @@
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         self.build_tree_contents([('a', '0123456789')],)
-        server = self.get_readonly_server()
-        self.transport = self._transport(server.get_url())
 
     def create_transport_readonly_server(self):
         return http_server.HttpServer(protocol_version=self._protocol_version)
 
     def _file_contents(self, relpath, ranges):
+        t = self.get_readonly_transport()
         offsets = [ (start, end - start + 1) for start, end in ranges]
-        coalesce = self.transport._coalesce_offsets
+        coalesce = t._coalesce_offsets
         coalesced = list(coalesce(offsets, limit=0, fudge_factor=0))
-        code, data = self.transport._get(relpath, coalesced)
+        code, data = t._get(relpath, coalesced)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         for start, end in ranges:
             data.seek(start)
             yield data.read(end - start + 1)
 
     def _file_tail(self, relpath, tail_amount):
-        code, data = self.transport._get(relpath, [], tail_amount)
+        t = self.get_readonly_transport()
+        code, data = t._get(relpath, [], tail_amount)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         data.seek(-tail_amount, 2)
         return data.read(tail_amount)
@@ -1313,28 +1281,17 @@
 class TestHTTPRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirection between http servers."""
 
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
-
     def setUp(self):
         super(TestHTTPRedirections, self).setUp()
         self.build_tree_contents([('a', '0123456789'),
                                   ('bundle',
                                   '# Bazaar revision bundle v0.9\n#\n')
                                   ],)
-        # The requests to the old server will be redirected to the new server
-        self.old_transport = self._transport(self.old_server.get_url())
 
     def test_redirected(self):
-        self.assertRaises(errors.RedirectRequested, self.old_transport.get, 'a')
-        t = self._transport(self.new_server.get_url())
-        self.assertEqual('0123456789', t.get('a').read())
+        self.assertRaises(errors.RedirectRequested,
+                          self.get_old_transport().get, 'a')
+        self.assertEqual('0123456789', self.get_new_transport().get('a').read())
 
 
 class RedirectedRequest(_urllib2_wrappers.Request):
@@ -1357,6 +1314,27 @@
     test.overrideAttr(_urllib2_wrappers, 'Request', RedirectedRequest)
 
 
+def cleanup_http_redirection_connections(test):
+    # Some sockets are opened but never seen by _urllib, so we trap them at
+    # the _urllib2_wrappers level to be able to clean them up.
+    def socket_disconnect(sock):
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except socket.error:
+            pass
+    def connect(connection):
+        test.http_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.http_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPConnection, 'connect', connect)
+    def connect(connection):
+        test.https_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.https_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPSConnection, 'connect', connect)
+
+
 class TestHTTPSilentRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirections.
 
@@ -1375,9 +1353,10 @@
         if (features.pycurl.available()
             and self._transport == PyCurlTransport):
             raise tests.TestNotApplicable(
-                "pycurl doesn't redirect silently annymore")
+                "pycurl doesn't redirect silently anymore")
         super(TestHTTPSilentRedirections, self).setUp()
         install_redirected_request(self)
+        cleanup_http_redirection_connections(self)
         self.build_tree_contents([('a','a'),
                                   ('1/',),
                                   ('1/a', 'redirected once'),
@@ -1391,27 +1370,18 @@
                                   ('5/a', 'redirected 5 times'),
                                   ],)
 
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server, redirections are defined in the tests"""
-        return http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-
     def test_one_redirection(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         new_prefix = 'http://%s:%s' % (self.new_server.host,
                                        self.new_server.port)
         self.old_server.redirections = \
             [('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
 
     def test_five_redirections(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         old_prefix = 'http://%s:%s' % (self.old_server.host,
                                        self.old_server.port)
         new_prefix = 'http://%s:%s' % (self.new_server.host,
@@ -1423,7 +1393,7 @@
             ('/4(.*)', r'%s/5\1' % (new_prefix), 301),
             ('(/[^/]+)', r'%s/1\1' % (old_prefix), 301),
             ]
-        self.assertEqual('redirected 5 times',t._perform(req).read())
+        self.assertEqual('redirected 5 times', t._perform(req).read())
 
 
 class TestDoCatchRedirections(http_utils.TestCaseWithRedirectedWebserver):
@@ -1432,14 +1402,15 @@
     def setUp(self):
         super(TestDoCatchRedirections, self).setUp()
         self.build_tree_contents([('a', '0123456789'),],)
-
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+        self.old_transport = self.get_old_transport()
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_no_redirection(self):
-        t = self._transport(self.new_server.get_url())
+        t = self.get_new_transport()
 
         # We use None for redirected so that we fail if redirected
         self.assertEqual('0123456789',
@@ -1449,10 +1420,10 @@
     def test_one_redirection(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            return redirected_t
 
         self.assertEqual('0123456789',
                          transport.do_catching_redirections(
@@ -1488,7 +1459,9 @@
                                   ('b', 'contents of b\n'),])
 
     def create_transport_readonly_server(self):
-        return self._auth_server(protocol_version=self._protocol_version)
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -1507,7 +1480,8 @@
         return url
 
     def get_user_transport(self, user, password):
-        return self._transport(self.get_user_url(user, password))
+        t = transport.get_transport(self.get_user_url(user, password))
+        return t
 
     def test_no_user(self):
         self.server.add_user('joe', 'foo')
@@ -1698,7 +1672,7 @@
 
     def get_user_transport(self, user, password):
         self._install_env({'all_proxy': self.get_user_url(user, password)})
-        return self._transport(self.server.get_url())
+        return TestAuth.get_user_transport(self, user, password)
 
     def _install_env(self, env):
         for name, value in env.iteritems():
@@ -1731,6 +1705,7 @@
         self.readfile = StringIO(socket_read_content)
         self.writefile = StringIO()
         self.writefile.close = lambda: None
+        self.close = lambda: None
 
     def makefile(self, mode='r', bufsize=None):
         if 'r' in mode:
@@ -1746,16 +1721,19 @@
         # We use the VFS layer as part of HTTP tunnelling tests.
         self._captureVar('BZR_NO_SMART_VFS', None)
         self.transport_readonly_server = http_utils.HTTPServerWithSmarts
+        self.http_server = self.get_readonly_server()
 
     def create_transport_readonly_server(self):
-        return http_utils.HTTPServerWithSmarts(
+        server = http_utils.HTTPServerWithSmarts(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def test_open_bzrdir(self):
         branch = self.make_branch('relpath')
-        http_server = self.get_readonly_server()
-        url = http_server.get_url() + 'relpath'
+        url = self.http_server.get_url() + 'relpath'
         bd = bzrdir.BzrDir.open(url)
+        self.addCleanup(bd.transport.disconnect)
         self.assertIsInstance(bd, _mod_remote.RemoteBzrDir)
 
     def test_bulk_data(self):
@@ -1763,8 +1741,7 @@
         # The 'readv' command in the smart protocol both sends and receives
         # bulk data, so we use that.
         self.build_tree(['data-file'])
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport(self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         # Since we provide the medium, the url below will be mostly ignored
         # during the test, as long as the path is '/'.
@@ -1778,15 +1755,14 @@
         post_body = 'hello\n'
         expected_reply_body = 'ok\x012\n'
 
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport(self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         response = medium.send_http_smart_request(post_body)
         reply_body = response.read()
         self.assertEqual(expected_reply_body, reply_body)
 
     def test_smart_http_server_post_request_handler(self):
-        httpd = self.get_readonly_server()._get_httpd()
+        httpd = self.http_server.server
 
         socket = SampleSocket(
             'POST /.bzr/smart %s \r\n' % self._protocol_version
@@ -1824,14 +1800,14 @@
 
     def test_probe_smart_server(self):
         """Test error handling against server refusing smart requests."""
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # No need to build a valid smart request here, the server will not even
         # try to interpret it.
         self.assertRaises(errors.SmartProtocolError,
                           t.get_smart_medium().send_http_smart_request,
                           'whatever')
 
+
 class Test_redirected_to(tests.TestCase):
 
     def test_redirected_to_subdir(self):
@@ -1890,7 +1866,7 @@
     line.
     """
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         tcs = self.server.test_case_server
         requestline = self.rfile.readline()
         headers = self.MessageClass(self.rfile, 0)
@@ -1964,7 +1940,9 @@
         self.addCleanup(self.server.stop_server)
 
     def get_transport(self):
-        return self._transport(self.server.get_url())
+        t = self._transport(self.server.get_url())
+        # FIXME: Needs cleanup -- vila 20100611
+        return t
 
     def assertActivitiesMatch(self):
         self.assertEqual(self.server.bytes_read,
@@ -2075,7 +2053,7 @@
 '''
         t = self.get_transport()
         # We must send a single line of body bytes, see
-        # PredefinedRequestHandler.handle_one_request
+        # PredefinedRequestHandler._handle_one_request
         code, f = t._post('abc def end-of-body\n')
         self.assertEqual('lalala whatever as long as itsssss\n', f.read())
         self.assertActivitiesMatch()
@@ -2115,18 +2093,6 @@
     _auth_server = http_utils.HTTPBasicAuthServer
     _transport = _urllib.HttpTransport_urllib
 
-    def create_transport_readonly_server(self):
-        return self._auth_server(protocol_version=self._protocol_version)
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
-
     def setUp(self):
         super(TestAuthOnRedirected, self).setUp()
         self.build_tree_contents([('a','a'),
@@ -2137,19 +2103,26 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.old_transport = self._transport(self.old_server.get_url())
+        self.old_transport = self.get_old_transport()
         self.new_server.add_user('joe', 'foo')
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+    def create_transport_readonly_server(self):
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_auth_on_redirected_via_do_catching_redirections(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            self.addCleanup(redirected_t.disconnect)
+            return redirected_t
 
         stdout = tests.StringIOWrapper()
         stderr = tests.StringIOWrapper()
@@ -2176,7 +2149,7 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
         # stdout should be empty, stderr will contains the prompts

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2010-07-04 06:22:11 +0000
+++ bzrlib/tests/test_lsprof.py	2010-07-05 08:10:03 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_msgeditor.py	2010-08-29 14:32:45 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import (
     commit,
+    config,
     errors,
     msgeditor,
     osutils,
@@ -236,46 +237,25 @@
         self.assertRaises((IOError, OSError), msgeditor.edit_commit_message, '')
 
     def test__get_editor(self):
-        # Test that _get_editor can return a decent list of items
-        bzr_editor = os.environ.get('BZR_EDITOR')
-        visual = os.environ.get('VISUAL')
-        editor = os.environ.get('EDITOR')
-        try:
-            os.environ['BZR_EDITOR'] = 'bzr_editor'
-            os.environ['VISUAL'] = 'visual'
-            os.environ['EDITOR'] = 'editor'
-
-            ensure_config_dir_exists()
-            f = open(config_filename(), 'wb')
-            f.write('editor = config_editor\n')
-            f.close()
-
-            editors = list(msgeditor._get_editor())
-            editors = [editor for (editor, cfg_src) in editors]
-
-            self.assertEqual(['bzr_editor', 'config_editor', 'visual',
-                              'editor'], editors[:4])
-
-            if sys.platform == 'win32':
-                self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
-            else:
-                self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano',
-                                  'joe'], editors[4:])
-
-        finally:
-            # Restore the environment
-            if bzr_editor is None:
-                del os.environ['BZR_EDITOR']
-            else:
-                os.environ['BZR_EDITOR'] = bzr_editor
-            if visual is None:
-                del os.environ['VISUAL']
-            else:
-                os.environ['VISUAL'] = visual
-            if editor is None:
-                del os.environ['EDITOR']
-            else:
-                os.environ['EDITOR'] = editor
+        os.environ['BZR_EDITOR'] = 'bzr_editor'
+        os.environ['VISUAL'] = 'visual'
+        os.environ['EDITOR'] = 'editor'
+
+        conf = config.GlobalConfig.from_string('editor = config_editor\n',
+                                               save=True)
+
+        editors = list(msgeditor._get_editor())
+        editors = [editor for (editor, cfg_src) in editors]
+
+        self.assertEqual(['bzr_editor', 'config_editor', 'visual', 'editor'],
+                         editors[:4])
+
+        if sys.platform == 'win32':
+            self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
+        else:
+            self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano', 'joe'],
+                             editors[4:])
+
 
     def test__run_editor_EACCES(self):
         """If running a configured editor raises EACESS, the user is warned."""

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/test_server.py	2010-08-24 16:07:33 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2010 Canonical Ltd
+# Copyright (C) 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
+import socket
+import SocketServer
+import select
+import sys
+import threading
+
+
 from bzrlib import (
+    osutils,
     transport,
     urlutils,
     )
@@ -22,7 +31,18 @@
     chroot,
     pathfilter,
     )
-from bzrlib.smart import server
+from bzrlib.smart import (
+    medium,
+    server,
+    )
+
+
+def debug_threads():
+    # FIXME: There is a dependency loop between bzrlib.tests and
+    # bzrlib.tests.test_server that needs to be fixed. In the mean time
+    # defining this function is enough for our needs. -- vila 20100611
+    from bzrlib import tests
+    return 'threads' in tests.selftest_debug_flags
 
 
 class TestServer(transport.Server):
@@ -223,23 +243,486 @@
         raise NotImplementedError
 
 
-class SmartTCPServer_for_testing(server.SmartTCPServer):
+class ThreadWithException(threading.Thread):
+    """A catching exception thread.
+
+    If an exception occurs during the thread execution, it's caught and
+    re-raised when the thread is joined().
+    """
+
+    def __init__(self, *args, **kwargs):
+        # There are cases where the calling thread must wait, yet, if an
+        # exception occurs, the event should be set so the caller is not
+        # blocked. The main example is a calling thread that want to wait for
+        # the called thread to be in a given state before continuing.
+        try:
+            event = kwargs.pop('event')
+        except KeyError:
+            # If the caller didn't pass a specific event, create our own
+            event = threading.Event()
+        super(ThreadWithException, self).__init__(*args, **kwargs)
+        self.set_ready_event(event)
+        self.exception = None
+        self.ignored_exceptions = None # see set_ignored_exceptions
+
+    # compatibility thunk for python-2.4 and python-2.5...
+    if sys.version_info < (2, 6):
+        name = property(threading.Thread.getName, threading.Thread.setName)
+
+    def set_ready_event(self, event):
+        """Set the ``ready`` event used to synchronize exception catching.
+
+        When the thread uses an event to synchronize itself with another thread
+        (setting it when the other thread can wake up from a ``wait`` call),
+        the event must be set after catching an exception or the other thread
+        will hang.
+
+        Some threads require multiple events and should set the relevant one
+        when appropriate.
+        """
+        self.ready = event
+
+    def set_ignored_exceptions(self, ignored):
+        """Declare which exceptions will be ignored.
+
+        :param ignored: Can be either:
+           - None: all exceptions will be raised,
+           - an exception class: the instances of this class will be ignored,
+           - a tuple of exception classes: the instances of any class of the
+             list will be ignored,
+           - a callable: that will be passed the exception object
+             and should return True if the exception should be ignored
+        """
+        if ignored is None:
+            self.ignored_exceptions = None
+        elif isinstance(ignored, (Exception, tuple)):
+            self.ignored_exceptions = lambda e: isinstance(e, ignored)
+        else:
+            self.ignored_exceptions = ignored
+
+    def run(self):
+        """Overrides Thread.run to capture any exception."""
+        self.ready.clear()
+        try:
+            try:
+                super(ThreadWithException, self).run()
+            except:
+                self.exception = sys.exc_info()
+        finally:
+            # Make sure the calling thread is released
+            self.ready.set()
+
+
+    def join(self, timeout=5):
+        """Overrides Thread.join to raise any exception caught.
+
+
+        Calling join(timeout=0) will raise the caught exception or return None
+        if the thread is still alive.
+
+        The default timeout is set to 5 and should expire only when a thread
+        serving a client connection is hung.
+        """
+        super(ThreadWithException, self).join(timeout)
+        if self.exception is not None:
+            exc_class, exc_value, exc_tb = self.exception
+            self.exception = None # The exception should be raised only once
+            if (self.ignored_exceptions is None
+                or not self.ignored_exceptions(exc_value)):
+                # Raise non ignored exceptions
+                raise exc_class, exc_value, exc_tb
+        if timeout and self.isAlive():
+            # The timeout expired without joining the thread, the thread is
+            # therefore stucked and that's a failure as far as the test is
+            # concerned. We used to hang here.
+
+            # FIXME: we need to kill the thread, but as far as the test is
+            # concerned, raising an assertion is too strong. On most of the
+            # platforms, this doesn't occur, so just mentioning the problem is
+            # enough for now -- vila 2010824
+            sys.stderr.write('thread %s hung\n' % (self.name,))
+            #raise AssertionError('thread %s hung' % (self.name,))
+
+    def pending_exception(self):
+        """Raise the caught exception.
+
+        This does nothing if no exception occurred.
+        """
+        self.join(timeout=0)
+
+
+class TestingTCPServerMixin:
+    """Mixin to support running SocketServer.TCPServer in a thread.
+
+    Tests are connecting from the main thread, the server has to be run in a
+    separate thread.
+    """
+
+    def __init__(self):
+        self.started = threading.Event()
+        self.serving = None
+        self.stopped = threading.Event()
+        # We collect the resources used by the clients so we can release them
+        # when shutting down
+        self.clients = []
+        self.ignored_exceptions = None
+
+    def server_bind(self):
+        self.socket.bind(self.server_address)
+        self.server_address = self.socket.getsockname()
+
+    def serve(self):
+        self.serving = True
+        self.stopped.clear()
+        # We are listening and ready to accept connections
+        self.started.set()
+        try:
+            while self.serving:
+                # Really a connection but the python framework is generic and
+                # call them requests
+                self.handle_request()
+            # Let's close the listening socket
+            self.server_close()
+        finally:
+            self.stopped.set()
+
+    def handle_request(self):
+        """Handle one request.
+
+        The python version swallows some socket exceptions and we don't use
+        timeout, so we override it to better control the server behavior.
+        """
+        request, client_address = self.get_request()
+        if self.verify_request(request, client_address):
+            try:
+                self.process_request(request, client_address)
+            except:
+                self.handle_error(request, client_address)
+                self.close_request(request)
+
+    def get_request(self):
+        return self.socket.accept()
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket ! This is useful when we
+        stop the server with a dummy last connection.
+        """
+        return self.serving
+
+    def handle_error(self, request, client_address):
+        # Stop serving and re-raise the last exception seen
+        self.serving = False
+        # The following can be used for debugging purposes, it will display the
+        # exception and the traceback just when it occurs instead of waiting
+        # for the thread to be joined.
+
+        # SocketServer.BaseServer.handle_error(self, request, client_address)
+        raise
+
+    def ignored_exceptions_during_shutdown(self, e):
+        if sys.platform == 'win32':
+            accepted_errnos = [errno.EBADF, errno.WSAEBADF, errno.WSAENOTCONN,
+                               errno.WSAECONNRESET, errno.WSAESHUTDOWN]
+        else:
+            accepted_errnos = [errno.EBADF, errno.ENOTCONN, errno.ECONNRESET]
+        if isinstance(e, socket.error) and e[0] in accepted_errnos:
+            return True
+        return False
+
+    # The following methods are called by the main thread
+
+    def stop_client_connections(self):
+        while self.clients:
+            c = self.clients.pop()
+            self.shutdown_client(c)
+
+    def shutdown_socket(self, sock):
+        """Properly shutdown a socket.
+
+        This should be called only when no other thread is trying to use the
+        socket.
+        """
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except Exception, e:
+            if self.ignored_exceptions(e):
+                pass
+            else:
+                raise
+
+    # The following methods are called by the main thread
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        self.ignored_exceptions = ignored_exceptions
+        thread.set_ignored_exceptions(self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        """Raise server uncaught exception.
+
+        Daughter classes can override this if they use daughter threads.
+        """
+        thread.pending_exception()
+
+
+class TestingTCPServer(TestingTCPServerMixin, SocketServer.TCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.TCPServer.__init__(self, server_address,
+                                        request_handler_class)
+
+    def get_request(self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        self.clients.append((sock, addr))
+        return sock, addr
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr = client
+        self.shutdown_socket(sock)
+
+
+class TestingThreadingTCPServer(TestingTCPServerMixin,
+                                SocketServer.ThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.ThreadingTCPServer.__init__(self, server_address,
+                                                 request_handler_class)
+
+    def get_request (self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        # The thread is not create yet, it will be updated in process_request
+        self.clients.append((sock, addr, None))
+        return sock, addr
+
+    def process_request_thread(self, started, stopped, request, client_address):
+        started.set()
+        SocketServer.ThreadingTCPServer.process_request_thread(
+            self, request, client_address)
+        self.close_request(request)
+        stopped.set()
+
+    def process_request(self, request, client_address):
+        """Start a new thread to process the request."""
+        started = threading.Event()
+        stopped = threading.Event()
+        t = ThreadWithException(
+            event=stopped,
+            name='%s -> %s' % (client_address, self.server_address),
+            target = self.process_request_thread,
+            args = (started, stopped, request, client_address))
+        # Update the client description
+        self.clients.pop()
+        self.clients.append((request, client_address, t))
+        # Propagate the exception handler since we must use the same one for
+        # connections running in their own threads than TestingTCPServer.
+        t.set_ignored_exceptions(self.ignored_exceptions)
+        t.start()
+        started.wait()
+        if debug_threads():
+            sys.stderr.write('Client thread %s started\n' % (t.name,))
+        # If an exception occured during the thread start, it will get raised.
+        t.pending_exception()
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr, connection_thread = client
+        self.shutdown_socket(sock)
+        if connection_thread is not None:
+            # The thread has been created only if the request is processed but
+            # after the connection is inited. This could happen during server
+            # shutdown. If an exception occurred in the thread it will be
+            # re-raised
+            if debug_threads():
+                sys.stderr.write('Client thread %s will be joined\n'
+                                 % (connection_thread.name,))
+            connection_thread.join()
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        TestingTCPServerMixin.set_ignored_exceptions(self, thread,
+                                                     ignored_exceptions)
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.set_ignored_exceptions(
+                    self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.pending_exception()
+        TestingTCPServerMixin._pending_exception(self, thread)
+
+
+class TestingTCPServerInAThread(transport.Server):
+    """A server in a thread that re-raise thread exceptions."""
+
+    def __init__(self, server_address, server_class, request_handler_class):
+        self.server_class = server_class
+        self.request_handler_class = request_handler_class
+        self.host, self.port = server_address
+        self.server = None
+        self._server_thread = None
+
+    def __repr__(self):
+        return "%s(%s:%s)" % (self.__class__.__name__, self.host, self.port)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class)
+
+    def start_server(self):
+        self.server = self.create_server()
+        self._server_thread = ThreadWithException(
+            event=self.server.started,
+            target=self.run_server)
+        self._server_thread.start()
+        # Wait for the server thread to start (i.e release the lock)
+        self.server.started.wait()
+        # Get the real address, especially the port
+        self.host, self.port = self.server.server_address
+        self._server_thread.name = self.server.server_address
+        if debug_threads():
+            sys.stderr.write('Server thread %s started\n'
+                             % (self._server_thread.name,))
+        # If an exception occured during the server start, it will get raised,
+        # otherwise, the server is blocked on its accept() call.
+        self._server_thread.pending_exception()
+        # From now on, we'll use a different event to ensure the server can set
+        # its exception
+        self._server_thread.set_ready_event(self.server.stopped)
+
+    def run_server(self):
+        self.server.serve()
+
+    def stop_server(self):
+        if self.server is None:
+            return
+        try:
+            # The server has been started successfully, shut it down now.  As
+            # soon as we stop serving, no more connection are accepted except
+            # one to get out of the blocking listen.
+            self.set_ignored_exceptions(
+                self.server.ignored_exceptions_during_shutdown)
+            self.server.serving = False
+            if debug_threads():
+                sys.stderr.write('Server thread %s will be joined\n'
+                                 % (self._server_thread.name,))
+            # The server is listening for a last connection, let's give it:
+            last_conn = None
+            try:
+                last_conn = osutils.connect_socket((self.host, self.port))
+            except socket.error, e:
+                # But ignore connection errors as the point is to unblock the
+                # server thread, it may happen that it's not blocked or even
+                # not started.
+                pass
+            # We start shutting down the client while the server itself is
+            # shutting down.
+            self.server.stop_client_connections()
+            # Now we wait for the thread running self.server.serve() to finish
+            self.server.stopped.wait()
+            if last_conn is not None:
+                # Close the last connection without trying to use it. The
+                # server will not process a single byte on that socket to avoid
+                # complications (SSL starts with a handshake for example).
+                last_conn.close()
+            # Check for any exception that could have occurred in the server
+            # thread
+            try:
+                self._server_thread.join()
+            except Exception, e:
+                if self.server.ignored_exceptions(e):
+                    pass
+                else:
+                    raise
+        finally:
+            # Make sure we can be called twice safely, note that this means
+            # that we will raise a single exception even if several occurred in
+            # the various threads involved.
+            self.server = None
+
+    def set_ignored_exceptions(self, ignored_exceptions):
+        """Install an exception handler for the server."""
+        self.server.set_ignored_exceptions(self._server_thread,
+                                           ignored_exceptions)
+
+    def pending_exception(self):
+        """Raise uncaught exception in the server."""
+        self.server._pending_exception(self._server_thread)
+
+
+class TestingSmartConnectionHandler(SocketServer.BaseRequestHandler,
+                                    medium.SmartServerSocketStreamMedium):
+
+    def __init__(self, request, client_address, server):
+        medium.SmartServerSocketStreamMedium.__init__(
+            self, request, server.backing_transport,
+            server.root_client_path)
+        request.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        SocketServer.BaseRequestHandler.__init__(self, request, client_address,
+                                                 server)
+
+    def handle(self):
+        while not self.finished:
+            server_protocol = self._build_protocol()
+            self._serve_one_request(server_protocol)
+
+
+class TestingSmartServer(TestingThreadingTCPServer, server.SmartTCPServer):
+
+    def __init__(self, server_address, request_handler_class,
+                 backing_transport, root_client_path):
+        TestingThreadingTCPServer.__init__(self, server_address,
+                                           request_handler_class)
+        server.SmartTCPServer.__init__(self, backing_transport,
+                                       root_client_path)
+    def serve(self):
+        # FIXME: No test are exercising the hooks for the test server
+        # -- vila 20100618
+        self.run_server_started_hooks()
+        try:
+            TestingThreadingTCPServer.serve(self)
+        finally:
+            self.run_server_stopped_hooks()
+
+    def get_url(self):
+        """Return the url of the server"""
+        return "bzr://%s:%d/" % self.server_address
+
+
+class SmartTCPServer_for_testing(TestingTCPServerInAThread):
     """Server suitable for use by transport tests.
 
     This server is backed by the process's cwd.
     """
-
     def __init__(self, thread_name_suffix=''):
-        super(SmartTCPServer_for_testing, self).__init__(None)
         self.client_path_extra = None
         self.thread_name_suffix = thread_name_suffix
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        return transport.get_transport(backing_transport_server.get_url())
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SmartTCPServer_for_testing, self).__init__(
+                (self.host, self.port),
+                TestingSmartServer,
+                TestingSmartConnectionHandler)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class,
+                                 self.backing_transport,
+                                 self.root_client_path)
+
 
     def start_server(self, backing_transport_server=None,
-              client_path_extra='/extra/'):
+                     client_path_extra='/extra/'):
         """Set up server for testing.
 
         :param backing_transport_server: backing server to use.  If not
@@ -254,6 +737,7 @@
         """
         if not client_path_extra.startswith('/'):
             raise ValueError(client_path_extra)
+        self.root_client_path = self.client_path_extra = client_path_extra
         from bzrlib.transport.chroot import ChrootServer
         if backing_transport_server is None:
             backing_transport_server = LocalURLServer()
@@ -262,15 +746,20 @@
         self.chroot_server.start_server()
         self.backing_transport = transport.get_transport(
             self.chroot_server.get_url())
-        self.root_client_path = self.client_path_extra = client_path_extra
-        self.start_background_thread(self.thread_name_suffix)
+        super(SmartTCPServer_for_testing, self).start_server()
 
     def stop_server(self):
-        self.stop_background_thread()
-        self.chroot_server.stop_server()
+        try:
+            super(SmartTCPServer_for_testing, self).stop_server()
+        finally:
+            self.chroot_server.stop_server()
+
+    def get_backing_transport(self, backing_transport_server):
+        """Get a backing transport from a server we are decorating."""
+        return transport.get_transport(backing_transport_server.get_url())
 
     def get_url(self):
-        url = super(SmartTCPServer_for_testing, self).get_url()
+        url = self.server.get_url()
         return url[:-1] + self.client_path_extra
 
     def get_bogus_url(self):

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_sftp_transport.py	2010-06-23 08:17:20 +0000
@@ -504,7 +504,7 @@
     """Test that AuthenticationConfig can supply default usernames."""
 
     def get_transport_for_connection(self, set_config):
-        port = self.get_server()._listener.port
+        port = self.get_server().port
         if set_config:
             conf = config.AuthenticationConfig()
             conf._get_config().update(

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2010-06-25 09:56:07 +0000
+++ bzrlib/tests/test_smart_transport.py	2010-07-01 15:25:41 +0000
@@ -40,7 +40,10 @@
         server,
         vfs,
 )
-from bzrlib.tests import test_smart
+from bzrlib.tests import (
+    test_smart,
+    test_server,
+    )
 from bzrlib.transport import (
         http,
         local,
@@ -972,24 +975,27 @@
             base = 'a_url'
             def external_url(self):
                 return self.base
-            def get_bytes(self, path):
+            def get(self, path):
                 raise Exception("some random exception from inside server")
-        smart_server = server.SmartTCPServer(backing_transport=FlakyTransport())
-        smart_server.start_background_thread('-' + self.id())
-        try:
-            transport = remote.RemoteTCPTransport(smart_server.get_url())
-            err = self.assertRaises(errors.UnknownErrorFromSmartServer,
-                transport.get, 'something')
-            self.assertContainsRe(str(err), 'some random exception')
-            transport.disconnect()
-        finally:
-            smart_server.stop_background_thread()
+
+        class FlakyServer(test_server.SmartTCPServer_for_testing):
+            def get_backing_transport(self, backing_transport_server):
+                return FlakyTransport()
+
+        smart_server = FlakyServer()
+        smart_server.start_server()
+        self.addCleanup(smart_server.stop_server)
+        t = remote.RemoteTCPTransport(smart_server.get_url())
+        self.addCleanup(t.disconnect)
+        err = self.assertRaises(errors.UnknownErrorFromSmartServer,
+                                t.get, 'something')
+        self.assertContainsRe(str(err), 'some random exception')
 
 
 class SmartTCPTests(tests.TestCase):
     """Tests for connection/end to end behaviour using the TCP server.
 
-    All of these tests are run with a server running on another thread serving
+    All of these tests are run with a server running in another thread serving
     a MemoryTransport, and a connection to it already open.
 
     the server is obtained by calling self.start_server(readonly=False).
@@ -1003,7 +1009,7 @@
         # NB: Tests using this fall into two categories: tests of the server,
         # tests wanting a server. The latter should be updated to use
         # self.vfs_transport_factory etc.
-        if not backing_transport:
+        if backing_transport is None:
             mem_server = memory.MemoryServer()
             mem_server.start_server()
             self.addCleanup(mem_server.stop_server)
@@ -1017,41 +1023,45 @@
             self.backing_transport = transport.get_transport(
                 "readonly+" + self.backing_transport.abspath('.'))
         self.server = server.SmartTCPServer(self.backing_transport)
+        self.server.start_server('127.0.0.1', 0)
         self.server.start_background_thread('-' + self.id())
         self.transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.addCleanup(self.tearDownServer)
+        self.addCleanup(self.stop_server)
         self.permit_url(self.server.get_url())
 
-    def tearDownServer(self):
+    def stop_server(self):
+        """Disconnect the client and stop the server.
+
+        This must be re-entrant as some tests will call it explicitly in
+        addition to the normal cleanup.
+        """
         if getattr(self, 'transport', None):
             self.transport.disconnect()
             del self.transport
         if getattr(self, 'server', None):
             self.server.stop_background_thread()
-            # XXX: why not .stop_server() -- mbp 20100106
             del self.server
 
 
 class TestServerSocketUsage(SmartTCPTests):
 
-    def test_server_setup_teardown(self):
-        """It should be safe to teardown the server with no requests."""
+    def test_server_start_stop(self):
+        """It should be safe to stop the server with no requests."""
         self.start_server()
-        server = self.server
-        transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.tearDownServer()
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(self.server.get_url())
+        self.stop_server()
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
     def test_server_closes_listening_sock_on_shutdown_after_request(self):
         """The server should close its listening socket when it's stopped."""
         self.start_server()
-        server = self.server
+        server_url = self.server.get_url()
         self.transport.has('.')
-        self.tearDownServer()
+        self.stop_server()
         # if the listening socket has closed, we should get a BADFD error
         # when connecting, rather than a hang.
-        transport = remote.RemoteTCPTransport(server.get_url())
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(server_url)
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
 
 class WritableEndToEndTests(SmartTCPTests):
@@ -1192,7 +1202,7 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 
@@ -1211,7 +1221,7 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2009-07-19 01:36:56 +0000
+++ bzrlib/tests/test_smtp_connection.py	2010-08-30 07:34:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2009 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -91,9 +91,7 @@
 class TestSMTPConnection(tests.TestCaseInTempDir):
 
     def get_connection(self, text, smtp_factory=None):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return smtp_connection.SMTPConnection(my_config,
                                               _smtp_factory=smtp_factory)
 
@@ -160,17 +158,18 @@
 
     def test_authenticate_with_byte_strings(self):
         user = 'joe'
-        password = 'h\xC3\xACspass'
+        unicode_pass = u'h\xECspass'
+        utf8_pass = unicode_pass.encode('utf-8')
         factory = WideOpenSMTPFactory()
         conn = self.get_connection(
-            '[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
-            % (user, password), smtp_factory=factory)
-        self.assertEqual(u'h\xECspass', conn._smtp_password)
+            u'[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
+            % (user, unicode_pass), smtp_factory=factory)
+        self.assertEqual(unicode_pass, conn._smtp_password)
         conn._connect()
         self.assertEqual([('connect', 'localhost'),
                           ('ehlo',),
                           ('has_extn', 'starttls'),
-                          ('login', user, password)], factory._calls)
+                          ('login', user, utf8_pass)], factory._calls)
         smtp_username, smtp_password = factory._calls[-1][1:]
         self.assertIsInstance(smtp_username, str)
         self.assertIsInstance(smtp_password, str)

=== added file 'bzrlib/tests/test_test_server.py'
--- bzrlib/tests/test_test_server.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_test_server.py	2010-06-30 16:24:01 +0000
@@ -0,0 +1,263 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import errno
+import socket
+import SocketServer
+import threading
+
+from bzrlib import (
+    osutils,
+    tests,
+    )
+from bzrlib.tests import test_server
+
+
+def load_tests(basic_tests, module, loader):
+    suite = loader.suiteClass()
+    server_tests, remaining_tests = tests.split_suite_by_condition(
+        basic_tests, tests.condition_isinstance(TestTCPServerInAThread))
+    server_scenarios = [ (name, {'server_class': getattr(test_server, name)})
+                         for name in
+                         ('TestingTCPServer', 'TestingThreadingTCPServer')]
+    tests.multiply_tests(server_tests, server_scenarios, suite)
+    suite.addTest(remaining_tests)
+    return suite
+
+
+class TestThreadWithException(tests.TestCase):
+
+    def test_start_and_join_smoke_test(self):
+        def do_nothing():
+            pass
+
+        tt = test_server.ThreadWithException(target=do_nothing)
+        tt.start()
+        tt.join()
+
+    def test_exception_is_re_raised(self):
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            raise MyException()
+
+        tt = test_server.ThreadWithException(target=raise_my_exception)
+        tt.start()
+        self.assertRaises(MyException, tt.join)
+
+    def test_join_when_no_exception(self):
+        resume = threading.Event()
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            # Wait for the test to tell us to resume
+            resume.wait()
+            # Now we can raise
+            raise MyException()
+
+        tt = test_server.ThreadWithException(target=raise_my_exception)
+        tt.start()
+        tt.join(timeout=0)
+        self.assertIs(None, tt.exception)
+        resume.set()
+        self.assertRaises(MyException, tt.join)
+
+
+class TCPClient(object):
+
+    def __init__(self):
+        self.sock = None
+
+    def connect(self, addr):
+        if self.sock is not None:
+            raise AssertionError('Already connected to %r'
+                                 % (self.sock.getsockname(),))
+        self.sock = osutils.connect_socket(addr)
+
+    def disconnect(self):
+        if self.sock is not None:
+            try:
+                self.sock.shutdown(socket.SHUT_RDWR)
+                self.sock.close()
+            except socket.error, e:
+                if e[0] in (errno.EBADF, errno.ENOTCONN):
+                    # Right, the socket is already down
+                    pass
+                else:
+                    raise
+            self.sock = None
+
+    def write(self, s):
+        return self.sock.sendall(s)
+
+    def read(self, bufsize=4096):
+        return self.sock.recv(bufsize)
+
+
+class TCPConnectionHandler(SocketServer.StreamRequestHandler):
+
+    def handle(self):
+        self.done = False
+        self.handle_connection()
+        while not self.done:
+            self.handle_connection()
+
+    def handle_connection(self):
+        req = self.rfile.readline()
+        if not req:
+            self.done = True
+        elif req == 'ping\n':
+            self.wfile.write('pong\n')
+        else:
+            raise ValueError('[%s] not understood' % req)
+
+
+class TestTCPServerInAThread(tests.TestCase):
+
+    # Set by load_tests()
+    server_class = None
+
+    def get_server(self, server_class=None, connection_handler_class=None):
+        if server_class is not None:
+            self.server_class = server_class
+        if connection_handler_class is None:
+            connection_handler_class = TCPConnectionHandler
+        server =  test_server.TestingTCPServerInAThread(
+            ('localhost', 0), self.server_class, connection_handler_class)
+        server.start_server()
+        self.addCleanup(server.stop_server)
+        return server
+
+    def get_client(self):
+        client = TCPClient()
+        self.addCleanup(client.disconnect)
+        return client
+
+    def get_server_connection(self, server, conn_rank):
+        return server.server.clients[conn_rank]
+
+    def assertClientAddr(self, client, server, conn_rank):
+        conn = self.get_server_connection(server, conn_rank)
+        self.assertEquals(client.sock.getsockname(), conn[1])
+
+    def test_start_stop(self):
+        server = self.get_server()
+        client = self.get_client()
+        server.stop_server()
+        # since the server doesn't accept connections anymore attempting to
+        # connect should fail
+        client = self.get_client()
+        self.assertRaises(socket.error,
+                          client.connect, (server.host, server.port))
+
+    def test_client_talks_server_respond(self):
+        server = self.get_server()
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        self.assertIs(None, client.write('ping\n'))
+        resp = client.read()
+        self.assertClientAddr(client, server, 0)
+        self.assertEquals('pong\n', resp)
+
+    def test_server_fails_to_start(self):
+        class CantStart(Exception):
+            pass
+
+        class CantStartServer(test_server.TestingTCPServer):
+
+            def server_bind(self):
+                raise CantStart()
+
+        # The exception is raised in the main thread
+        self.assertRaises(CantStart,
+                          self.get_server, server_class=CantStartServer)
+
+    def test_server_fails_while_serving_or_stopping(self):
+        class CantConnect(Exception):
+            pass
+
+        class FailingConnectionHandler(TCPConnectionHandler):
+
+            def handle(self):
+                raise CantConnect()
+
+        server = self.get_server(
+            connection_handler_class=FailingConnectionHandler)
+        # The server won't fail until a client connect
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        try:
+            # Now we must force the server to answer by sending the request and
+            # waiting for some answer. But since we don't control when the
+            # server thread will be given cycles, we don't control either
+            # whether our reads or writes may hang.
+            client.sock.settimeout(0.1)
+            client.write('ping\n')
+            client.read()
+        except socket.error:
+            pass
+        # Now the server has raised the exception in its own thread
+        self.assertRaises(CantConnect, server.stop_server)
+
+    def test_server_crash_while_responding(self):
+        sync = threading.Event()
+        sync.clear()
+        class FailToRespond(Exception):
+            pass
+
+        class FailingDuringResponseHandler(TCPConnectionHandler):
+
+            def handle_connection(self):
+                req = self.rfile.readline()
+                threading.currentThread().set_ready_event(sync)
+                raise FailToRespond()
+
+        server = self.get_server(
+            connection_handler_class=FailingDuringResponseHandler)
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        client.write('ping\n')
+        sync.wait()
+        self.assertRaises(FailToRespond, server.pending_exception)
+
+    def test_exception_swallowed_while_serving(self):
+        sync = threading.Event()
+        sync.clear()
+        class CantServe(Exception):
+            pass
+
+        class FailingWhileServingConnectionHandler(TCPConnectionHandler):
+
+            def handle(self):
+                # We want to sync with the thread that is serving the
+                # connection.
+                threading.currentThread().set_ready_event(sync)
+                raise CantServe()
+
+        server = self.get_server(
+            connection_handler_class=FailingWhileServingConnectionHandler)
+        # Install the exception swallower
+        server.set_ignored_exceptions(CantServe)
+        client = self.get_client()
+        # Connect to the server so the exception is raised there
+        client.connect((server.host, server.port))
+        # Wait for the exception to propagate.
+        sync.wait()
+        # The connection wasn't served properly but the exception should have
+        # been swallowed.
+        server.pending_exception()

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_transport.py	2010-08-30 21:27:50 +0000
@@ -900,6 +900,8 @@
 
         bzr+ssh:// should cause bzr to run a remote bzr smart server over SSH.
         """
+        raise tests.TestSkipped('this test was recently broken,'
+                                ' see bug #626876')
         # This test actually causes a bzr instance to be invoked, which is very
         # expensive: it should be the only such test in the test suite.
         # A reasonable evolution for this would be to simply check inside
@@ -955,8 +957,12 @@
         ssh_server = stub_sftp.SFTPFullAbsoluteServer(StubSSHServer)
         # We *don't* want to override the default SSH vendor: the detected one
         # is the one to use.
+
+        # FIXME: I don't understand the above comment, SFTPFullAbsoluteServer
+        # inherits from SFTPServer which forces the SSH vendor to
+        # ssh.ParamikoVendor(). So it's forced, not detected. --vila 20100623
         self.start_server(ssh_server)
-        port = ssh_server._listener.port
+        port = ssh_server.port
 
         if sys.platform == 'win32':
             bzr_remote_path = sys.executable + ' ' + self.get_bzr_path()

=== modified file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2010-08-29 18:29:22 +0000
@@ -86,6 +86,15 @@
         # and it should be new member time in the stream.
         self.failUnless(myfile._new_member)
 
+    def test_negative_crc(self):
+        """Content with a negative crc should not break when written"""
+        sio = StringIO()
+        gfile = tuned_gzip.GzipFile(mode="w", fileobj=sio)
+        gfile.write("\xFF")
+        gfile.close()
+        self.assertEqual(gfile.crc & 0xFFFFFFFFL, 0xFF000000L)
+        self.assertEqual(sio.getvalue()[-8:-4], "\x00\x00\x00\xFF")
+
 
 class TestToGzip(TestCase):
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2010-06-02 01:45:31 +0000
+++ bzrlib/transport/__init__.py	2010-06-18 15:38:20 +0000
@@ -1286,6 +1286,12 @@
         # should be asked to ConnectedTransport only.
         return None
 
+    def disconnect(self):
+        # This is really needed for ConnectedTransport only, but it's easier to
+        # have Transport do nothing than testing that the disconnect should be
+        # asked to ConnectedTransport only.
+        pass
+
     def _redirected_to(self, source, target):
         """Returns a transport suitable to re-issue a redirected request.
 
@@ -1550,6 +1556,13 @@
             transport = self.__class__(other_base, _from_transport=self)
         return transport
 
+    def disconnect(self):
+        """Disconnect the transport.
+
+        If and when required the transport willl reconnect automatically.
+        """
+        raise NotImplementedError(self.disconnect)
+
 
 def get_transport(base, possible_transports=None):
     """Open a transport to access a URL or directory.

=== modified file 'bzrlib/transport/ftp/__init__.py'
--- bzrlib/transport/ftp/__init__.py	2010-08-25 10:21:17 +0000
+++ bzrlib/transport/ftp/__init__.py	2010-08-30 07:42:12 +0000
@@ -170,6 +170,11 @@
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _translate_ftp_error(self, err, path, extra=None,
                               unknown_exc=FtpPathError):
         """Try to translate an ftplib exception to a bzrlib exception.

=== modified file 'bzrlib/transport/ftp/_gssapi.py'
--- bzrlib/transport/ftp/_gssapi.py	2010-05-30 16:26:11 +0000
+++ bzrlib/transport/ftp/_gssapi.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/transport/gio_transport.py'
--- bzrlib/transport/gio_transport.py	2010-08-25 10:21:17 +0000
+++ bzrlib/transport/gio_transport.py	2010-08-30 07:42:12 +0000
@@ -241,7 +241,13 @@
                                         " %s" % str(e), orig_error=e)
         return connection, (user, password)
 
+    def disconnect(self):
+        # FIXME: Nothing seems to be necessary here, which sounds a bit strange
+        # -- vila 20100601
+        pass
+
     def _reconnect(self):
+        # FIXME: This doesn't seem to be used -- vila 20100601
         """Create a new connection with the previously used credentials"""
         credentials = self._get_credentials()
         connection, credentials = self._create_connection(credentials)

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/__init__.py	2010-06-01 13:01:20 +0000
@@ -629,6 +629,11 @@
         """
         pass
 
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        t = self._http_transport_ref()
+        t.disconnect()
+
 
 # TODO: May be better located in smart/medium.py with the other
 # SmartMediumRequest classes

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2010-05-30 16:22:32 +0000
+++ bzrlib/transport/http/_pycurl.py	2010-06-01 13:01:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -128,6 +128,11 @@
             self._set_connection(connection, auth)
         return connection
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def has(self, relpath):
         """See Transport.has()"""
         # We set NO BODY=0 in _get_full, so it should be safe

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/_urllib.py	2010-06-01 13:01:20 +0000
@@ -99,6 +99,11 @@
 
         return response
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get(self, relpath, offsets, tail_amount=0):
         """See HttpTransport._get"""
         abspath = self._remote_path(relpath)

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2010-06-17 09:23:19 +0000
+++ bzrlib/transport/remote.py	2010-06-18 15:38:20 +0000
@@ -435,7 +435,9 @@
         remote._translate_error(err, path=relpath)
 
     def disconnect(self):
-        self.get_smart_medium().disconnect()
+        m = self.get_smart_medium()
+        if m is not None:
+            m.disconnect()
 
     def stat(self, relpath):
         resp = self._call2('stat', self._remote_path(relpath))

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2010-07-19 10:55:15 +0000
+++ bzrlib/transport/sftp.py	2010-08-24 13:03:18 +0000
@@ -389,6 +389,11 @@
                                          self._host, self._port)
         return connection, (user, password)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get_sftp(self):
         """Ensures that a connection is established"""
         connection = self._get_connection()

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tuned_gzip.py	2010-08-29 18:30:45 +0000
@@ -395,4 +395,12 @@
         # (4 seconds to 1 seconds for the sample upgrades I was testing).
         self.write(''.join(lines))
 
+    if sys.version_info > (2, 7):
+        # As of Python 2.7 the crc32 must be positive when close is called
+        def close(self):
+            if self.fileobj is None:
+                return
+            if self.mode == gzip.WRITE:
+                self.crc &= 0xFFFFFFFFL
+            gzip.GzipFile.close(self)
 

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2010-08-16 06:46:17 +0000
+++ bzrlib/versionedfile.py	2010-08-28 15:34:24 +0000
@@ -46,7 +46,6 @@
 from bzrlib.transport.memory import MemoryTransport
 """)
 from bzrlib.registry import Registry
-from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
 from bzrlib import bencode
 

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2009-09-20 22:12:36 +0000
+++ bzrlib/weavefile.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'doc/ja/user-reference/index.txt'
--- doc/ja/user-reference/index.txt	2010-08-13 19:08:57 +0000
+++ doc/ja/user-reference/index.txt	2010-08-30 14:23:23 +0000
@@ -40,7 +40,7 @@
 
 次のウェブサイトはBazaarに関する詳細な情報を提供します:
 
-:ホームページ:                     http://www.bazaar.canonical.com/
+:ホームページ:                     http://bazaar.canonical.com/
 :公式ドキュメント:                 http://doc.bazaar.canonical.com/
 :Launchpad:                     https://launchpad.net/bzr/
 

